import{cl as t,cn as a}from"./D8zG9Ut8.js";const i=t("scanners",{state:()=>({scanners:[]}),actions:{addOrUpdateScanner(e){const s=this.scanners.findIndex(n=>n.endpoint===e.endpoint);s===-1?this.scanners.push(e):this.scanners[s]=e},deleteScanner(e){this.scanners=this.scanners.filter(s=>s.endpoint!==e)}},persist:{storage:a.localStorage}});export{i as u};
