.v-checkbox.v-input{flex:0 1 auto}.v-checkbox .v-selection-control{min-height:var(--v-input-control-height)}.create-scanner[data-v-5fbf6fc2]{cursor:pointer;transition:background-color .3s ease,box-shadow .3s ease}.create-scanner[data-v-5fbf6fc2]:hover{background-color:#f0f0f0;box-shadow:0 1px 2px #0000001a}.custom-option[data-v-5fbf6fc2]{margin:-22px auto}
