import{d as s,u as n,t as u,v as m,z as t,Z as l,A as e,B as p}from"./D8zG9Ut8.js";import{b as i,a as d,V as c,D as f}from"./CThvs1cs.js";import{A as _}from"./BE_3-dZR.js";import"./DlAUqK2U.js";import"./CcWp78UG.js";import"./C09w9mqt.js";import"./B_NqmWCc.js";import"./F47YBxX3.js";import"./DD_172SU.js";import"./BvH6OxSR.js";import"./Dbpq9kDR.js";const k=s({__name:"new",setup(g){const r=n(),a=[{text:"AGENT GROUPS",disabled:!1,to:"/agent-groups/list",exact:!0},{text:"New",disabled:!0,to:"/agent-groups/new",exact:!0}];function o(){r.push({path:"/agent-groups/list"})}return(x,b)=>(u(),m("div",null,[t(l(f),{breadcrumbs:a,class:"mb-5"}),t(c,{variant:"outlined",class:"my-4"},{default:e(()=>[t(i,null,{default:e(()=>[p(" Add a new Agent group ")]),_:1}),t(d,null,{default:e(()=>[t(_,{onCloseForm:o})]),_:1})]),_:1})]))}});export{k as default};
