import{d as n,N as i,A as o,t as c,z as r,W as l,B as s,y as t,ab as d,T as p,x as u}from"./D8zG9Ut8.js";import{_ as m}from"./DlAUqK2U.js";import{V as f}from"./CcWp78UG.js";const g=n({name:"DfScanProgress",props:{progress:{type:String,default:void 0},messageStatus:{type:String,default:void 0}},computed:{iconColor(){if(this.progress)switch(this.progress.toLowerCase()){case"done":return{icon:"mdi-check",color:"success",text:"Done"};case"in_progress":return{icon:"mdi-autorenew",color:"info",text:"Running"};case"not_started":return{icon:"mdi-timeline-plus-outline",color:"orange-darken-2",text:"Queued"};case"stopped":return{icon:"mdi-stop-circle",color:"red",text:"Stopped"};case"error":return{icon:"mdi-exclamation-thick",color:"purple",text:"Error"};case"disabled":return{icon:"mdi-stop-circle-outline",color:"grey",text:"Stopped"}}return{icon:"mdi-exclamation",color:"warning",text:""}}}});function x(e,S,h,_,C,b){return c(),i(f,{bottom:"",disabled:!e.messageStatus},{activator:o(({props:a})=>[r(p,d({color:e.iconColor.color,size:"small",label:"",variant:"flat"},a),{default:o(()=>[r(l,{size:"small",start:"",color:"white",class:"mb-n1"},{default:o(()=>[s(t(e.iconColor.icon),1)]),_:1}),s(" "+t(e.iconColor.text),1)]),_:2},1040,["color"])]),default:o(()=>[u("span",null,t(e.messageStatus),1)]),_:1},8,["disabled"])}const y=m(g,[["render",x]]);export{y as D};
