.cursor-pointer{cursor:pointer}#vulnerability-detail-container .v-dialog{display:block}#vulnerability-detail-container .v-dialog>.v-overlay__content{margin-right:3px;margin-top:3px;position:absolute;right:0}.v-data-table td[data-v-e8c5b85a]{cursor:pointer}
