.create-scanner[data-v-93550484]{cursor:pointer;transition:background-color .3s ease,box-shadow .3s ease}.create-scanner[data-v-93550484]:hover{background-color:#f0f0f0;box-shadow:0 1px 2px #0000001a}
