import{d as a,o as n,m as s,t,v as r,z as c,C as d,D as p,x as _}from"./D8zG9Ut8.js";import{u as i}from"./B_NqmWCc.js";import{S as l}from"./D-OV1_gf.js";import{_ as m}from"./DlAUqK2U.js";const f=e=>(d("data-v-f6b6d8b3"),e=e(),p(),e),u=f(()=>_("div",{class:"oxo-loader-bar"},null,-1)),S=a({__name:"index",setup(e){const o=i().scanners;return n(()=>{o.length===0?s("/scanners"):s("/scan/list")}),(x,b)=>(t(),r("div",null,[c(l),u]))}}),k=m(S,[["__scopeId","data-v-f6b6d8b3"]]);export{k as default};
