# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/scanner/proto/assets/network.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ostorlab.scanner.proto.assets import ip_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_ip__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+ostorlab/scanner/proto/assets/network.proto\x12%ostorlab.scanner.proto.assets.network\x1a&ostorlab/scanner/proto/assets/ip.proto\"A\n\x07Message\x12\x36\n\x03ips\x18\x01 \x03(\x0b\x32).ostorlab.scanner.proto.assets.ip.Message')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.scanner.proto.assets.network_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MESSAGE._serialized_start=126
  _MESSAGE._serialized_end=191
# @@protoc_insertion_point(module_scope)
