# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/report/vulnerability/vulnerability.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ostorlab.agent.message.proto.v3.asset.domain_name import domain_name_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_domain__name_dot_domain__name__pb2
from ostorlab.agent.message.proto.v3.asset.ip.v4 import v4_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v4_dot_v4__pb2
from ostorlab.agent.message.proto.v3.asset.ip.v6 import v6_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v6_dot_v6__pb2
from ostorlab.agent.message.proto.v3.asset.link import link_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_link_dot_link__pb2
from ostorlab.agent.message.proto.v3.asset.store.android_store import android_store_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_android__store_dot_android__store__pb2
from ostorlab.agent.message.proto.v3.asset.store.ios_store import ios_store_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_ios__store_dot_ios__store__pb2
from ostorlab.agent.message.proto.v3.asset.file import file_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_file__pb2
from ostorlab.agent.message.proto.v3.asset.file.android.aab import aab_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_aab_dot_aab__pb2
from ostorlab.agent.message.proto.v3.asset.file.android.apk import apk_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_apk_dot_apk__pb2
from ostorlab.agent.message.proto.v3.asset.file.ios.ipa import ipa_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_ios_dot_ipa_dot_ipa__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ostorlab/agent/message/proto/v3/report/vulnerability/vulnerability.proto',
  package='ostorlab.agent.message.proto.v3.report.vulnerability',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\nHostorlab/agent/message/proto/v3/report/vulnerability/vulnerability.proto\x12\x34ostorlab.agent.message.proto.v3.report.vulnerability\x1a\x43ostorlab/agent/message/proto/v3/asset/domain_name/domain_name.proto\x1a\x34ostorlab/agent/message/proto/v3/asset/ip/v4/v4.proto\x1a\x34ostorlab/agent/message/proto/v3/asset/ip/v6/v6.proto\x1a\x35ostorlab/agent/message/proto/v3/asset/link/link.proto\x1aMostorlab/agent/message/proto/v3/asset/store/android_store/android_store.proto\x1a\x45ostorlab/agent/message/proto/v3/asset/store/ios_store/ios_store.proto\x1a\x35ostorlab/agent/message/proto/v3/asset/file/file.proto\x1a@ostorlab/agent/message/proto/v3/asset/file/android/aab/aab.proto\x1a@ostorlab/agent/message/proto/v3/asset/file/android/apk/apk.proto\x1a<ostorlab/agent/message/proto/v3/asset/file/ios/ipa/ipa.proto\"\'\n\tReference\x12\r\n\x05title\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\"o\n\x08Metadata\x12\r\n\x05value\x18\x01 \x01(\t\x12T\n\x04type\x18\x03 \x01(\x0e\x32\x46.ostorlab.agent.message.proto.v3.report.vulnerability.MetadataTypeEnum\"\x8d\x07\n\x15VulnerabilityLocation\x12Q\n\x0b\x64omain_name\x18\x01 \x01(\x0b\x32:.ostorlab.agent.message.proto.v3.asset.domain_name.MessageH\x00\x12\x44\n\x04ipv4\x18\x02 \x01(\x0b\x32\x34.ostorlab.agent.message.proto.v3.asset.ip.v4.MessageH\x00\x12\x44\n\x04ipv6\x18\x03 \x01(\x0b\x32\x34.ostorlab.agent.message.proto.v3.asset.ip.v6.MessageH\x00\x12\x43\n\x04link\x18\x04 \x01(\x0b\x32\x33.ostorlab.agent.message.proto.v3.asset.link.MessageH\x00\x12[\n\randroid_store\x18\x05 \x01(\x0b\x32\x42.ostorlab.agent.message.proto.v3.asset.store.android_store.MessageH\x00\x12S\n\tios_store\x18\x06 \x01(\x0b\x32>.ostorlab.agent.message.proto.v3.asset.store.ios_store.MessageH\x00\x12\x43\n\x04\x66ile\x18\x07 \x01(\x0b\x32\x33.ostorlab.agent.message.proto.v3.asset.file.MessageH\x00\x12V\n\x0b\x61ndroid_aab\x18\x08 \x01(\x0b\x32?.ostorlab.agent.message.proto.v3.asset.file.android.aab.MessageH\x00\x12V\n\x0b\x61ndroid_apk\x18\t \x01(\x0b\x32?.ostorlab.agent.message.proto.v3.asset.file.android.apk.MessageH\x00\x12N\n\x07ios_ipa\x18\n \x01(\x0b\x32;.ostorlab.agent.message.proto.v3.asset.file.ios.ipa.MessageH\x00\x12P\n\x08metadata\x18\x64 \x03(\x0b\x32>.ostorlab.agent.message.proto.v3.report.vulnerability.MetadataB\x07\n\x05\x61sset\"0\n\rCategoryGroup\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\ncategories\x18\x02 \x03(\t\"\x82\x06\n\x07Message\x12\r\n\x05title\x18\x01 \x01(\t\x12U\n\x0brisk_rating\x18\x02 \x01(\x0e\x32@.ostorlab.agent.message.proto.v3.report.vulnerability.RiskRating\x12\x18\n\x10technical_detail\x18\x03 \x01(\t\x12\x19\n\x11short_description\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x16\n\x0erecommendation\x18\x06 \x01(\t\x12S\n\nreferences\x18\x07 \x03(\x0b\x32?.ostorlab.agent.message.proto.v3.report.vulnerability.Reference\x12\x1d\n\x0esecurity_issue\x18\x08 \x01(\x08:\x05\x66\x61lse\x12\x1c\n\rprivacy_issue\x18\t \x01(\x08:\x05\x66\x61lse\x12!\n\x12has_public_exploit\x18\n \x01(\x08:\x05\x66\x61lse\x12\"\n\x13targeted_by_malware\x18\x0b \x01(\x08:\x05\x66\x61lse\x12%\n\x16targeted_by_ransomware\x18\x0c \x01(\x08:\x05\x66\x61lse\x12\'\n\x18targeted_by_nation_state\x18\r \x01(\x08:\x05\x66\x61lse\x12\x0b\n\x03\x64na\x18\x0e \x01(\t\x12\x16\n\x0e\x63vss_v3_vector\x18\x0f \x01(\t\x12k\n\x16vulnerability_location\x18\x10 \x01(\x0b\x32K.ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation\x12\\\n\x0f\x63\x61tegory_groups\x18\x11 \x03(\x0b\x32\x43.ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup\x12\x16\n\x0e\x63vss_v4_vector\x18\x12 \x01(\t*~\n\nRiskRating\x12\x08\n\x04HIGH\x10\x00\x12\n\n\x06MEDIUM\x10\x01\x12\x07\n\x03LOW\x10\x02\x12\x0f\n\x0bPOTENTIALLY\x10\x03\x12\r\n\tHARDENING\x10\x04\x12\n\n\x06SECURE\x10\x05\x12\r\n\tIMPORTANT\x10\x06\x12\x08\n\x04INFO\x10\x07\x12\x0c\n\x08\x43RITICAL\x10\x08*P\n\x10MetadataTypeEnum\x12\r\n\tFILE_PATH\x10\x00\x12\x11\n\rCODE_LOCATION\x10\x01\x12\x07\n\x03URL\x10\x02\x12\x08\n\x04PORT\x10\x03\x12\x07\n\x03LOG\x10\x04'
  ,
  dependencies=[ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_domain__name_dot_domain__name__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v4_dot_v4__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v6_dot_v6__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_link_dot_link__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_android__store_dot_android__store__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_ios__store_dot_ios__store__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_file__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_aab_dot_aab__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_apk_dot_apk__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_ios_dot_ipa_dot_ipa__pb2.DESCRIPTOR,])

_RISKRATING = _descriptor.EnumDescriptor(
  name='RiskRating',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.RiskRating',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='HIGH', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIUM', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOW', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POTENTIALLY', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HARDENING', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IMPORTANT', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2650,
  serialized_end=2776,
)
_sym_db.RegisterEnumDescriptor(_RISKRATING)

RiskRating = enum_type_wrapper.EnumTypeWrapper(_RISKRATING)
_METADATATYPEENUM = _descriptor.EnumDescriptor(
  name='MetadataTypeEnum',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.MetadataTypeEnum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FILE_PATH', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CODE_LOCATION', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='URL', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PORT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOG', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2778,
  serialized_end=2858,
)
_sym_db.RegisterEnumDescriptor(_METADATATYPEENUM)

MetadataTypeEnum = enum_type_wrapper.EnumTypeWrapper(_METADATATYPEENUM)
HIGH = 0
MEDIUM = 1
LOW = 2
POTENTIALLY = 3
HARDENING = 4
SECURE = 5
IMPORTANT = 6
INFO = 7
CRITICAL = 8
FILE_PATH = 0
CODE_LOCATION = 1
URL = 2
PORT = 3
LOG = 4



_REFERENCE = _descriptor.Descriptor(
  name='Reference',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Reference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Reference.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='url', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Reference.url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=761,
  serialized_end=800,
)


_METADATA = _descriptor.Descriptor(
  name='Metadata',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Metadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Metadata.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Metadata.type', index=1,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=802,
  serialized_end=913,
)


_VULNERABILITYLOCATION = _descriptor.Descriptor(
  name='VulnerabilityLocation',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='domain_name', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.domain_name', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ipv4', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ipv4', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ipv6', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ipv6', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='link', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.link', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='android_store', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.android_store', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ios_store', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ios_store', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.file', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='android_aab', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.android_aab', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='android_apk', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.android_apk', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ios_ipa', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ios_ipa', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.metadata', index=10,
      number=100, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='asset', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.asset',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=916,
  serialized_end=1825,
)


_CATEGORYGROUP = _descriptor.Descriptor(
  name='CategoryGroup',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='categories', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup.categories', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1827,
  serialized_end=1875,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='risk_rating', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.risk_rating', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='technical_detail', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.technical_detail', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='short_description', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.short_description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recommendation', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.recommendation', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='references', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.references', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_issue', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.security_issue', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='privacy_issue', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.privacy_issue', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='has_public_exploit', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.has_public_exploit', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='targeted_by_malware', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.targeted_by_malware', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='targeted_by_ransomware', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.targeted_by_ransomware', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='targeted_by_nation_state', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.targeted_by_nation_state', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dna', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.dna', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cvss_v3_vector', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.cvss_v3_vector', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vulnerability_location', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.vulnerability_location', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='category_groups', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.category_groups', index=16,
      number=17, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cvss_v4_vector', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.cvss_v4_vector', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1878,
  serialized_end=2648,
)

_METADATA.fields_by_name['type'].enum_type = _METADATATYPEENUM
_VULNERABILITYLOCATION.fields_by_name['domain_name'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_domain__name_dot_domain__name__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ipv4'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v4_dot_v4__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ipv6'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v6_dot_v6__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['link'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_link_dot_link__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['android_store'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_android__store_dot_android__store__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ios_store'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_ios__store_dot_ios__store__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['file'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_file__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['android_aab'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_aab_dot_aab__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['android_apk'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_apk_dot_apk__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ios_ipa'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_ios_dot_ipa_dot_ipa__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['metadata'].message_type = _METADATA
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['domain_name'])
_VULNERABILITYLOCATION.fields_by_name['domain_name'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ipv4'])
_VULNERABILITYLOCATION.fields_by_name['ipv4'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ipv6'])
_VULNERABILITYLOCATION.fields_by_name['ipv6'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['link'])
_VULNERABILITYLOCATION.fields_by_name['link'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['android_store'])
_VULNERABILITYLOCATION.fields_by_name['android_store'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ios_store'])
_VULNERABILITYLOCATION.fields_by_name['ios_store'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['file'])
_VULNERABILITYLOCATION.fields_by_name['file'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['android_aab'])
_VULNERABILITYLOCATION.fields_by_name['android_aab'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['android_apk'])
_VULNERABILITYLOCATION.fields_by_name['android_apk'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ios_ipa'])
_VULNERABILITYLOCATION.fields_by_name['ios_ipa'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_MESSAGE.fields_by_name['risk_rating'].enum_type = _RISKRATING
_MESSAGE.fields_by_name['references'].message_type = _REFERENCE
_MESSAGE.fields_by_name['vulnerability_location'].message_type = _VULNERABILITYLOCATION
_MESSAGE.fields_by_name['category_groups'].message_type = _CATEGORYGROUP
DESCRIPTOR.message_types_by_name['Reference'] = _REFERENCE
DESCRIPTOR.message_types_by_name['Metadata'] = _METADATA
DESCRIPTOR.message_types_by_name['VulnerabilityLocation'] = _VULNERABILITYLOCATION
DESCRIPTOR.message_types_by_name['CategoryGroup'] = _CATEGORYGROUP
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.enum_types_by_name['RiskRating'] = _RISKRATING
DESCRIPTOR.enum_types_by_name['MetadataTypeEnum'] = _METADATATYPEENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Reference = _reflection.GeneratedProtocolMessageType('Reference', (_message.Message,), {
  'DESCRIPTOR' : _REFERENCE,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.Reference)
  })
_sym_db.RegisterMessage(Reference)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), {
  'DESCRIPTOR' : _METADATA,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.Metadata)
  })
_sym_db.RegisterMessage(Metadata)

VulnerabilityLocation = _reflection.GeneratedProtocolMessageType('VulnerabilityLocation', (_message.Message,), {
  'DESCRIPTOR' : _VULNERABILITYLOCATION,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation)
  })
_sym_db.RegisterMessage(VulnerabilityLocation)

CategoryGroup = _reflection.GeneratedProtocolMessageType('CategoryGroup', (_message.Message,), {
  'DESCRIPTOR' : _CATEGORYGROUP,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup)
  })
_sym_db.RegisterMessage(CategoryGroup)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.Message)
  })
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)
