.tab-header[data-v-f579388a]{border-bottom:1px solid #dadada;padding:.8rem}.data-table[data-v-f579388a]{height:100%;overflow-y:auto}.chip-container[data-v-f579388a]{display:flex;flex-wrap:wrap}.v-chip[data-v-f579388a]{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.v-chip__content[data-v-f579388a]{align-items:center;display:flex;justify-content:center}
