var st=Object.defineProperty;var rt=(e,n,t)=>n in e?st(e,n,{enumerable:!0,configurable:!0,writable:!0,value:t}):e[n]=t;var be=(e,n,t)=>rt(e,typeof n!="symbol"?n+"":n,t);import{d as H,i as N,N as T,A as D,t as v,x as R,v as I,y as M,ab as Z,aj as ot,z as F,ak as Me,al as lt,am as ut,an as j,W as oe,B as le,O as z,Y as Be,R as V,a as dt,o as ct,ao as pt,Z as gt,U as J,$ as de,a1 as qe,ap as ft,a3 as ce,aq as ht,r as Fe,j as ue,a7 as pe,ar as mt,as as bt,at as yt,au as vt,av as wt,a4 as He,aw as Te,ax as kt,ay as Ie,az as _t,aA as St,aB as xt,aC as $t,aD as At,aE as G,aF as Et,aG as Ft,aH as De,aI as Tt,aJ as It,S as X,ah as Ce,ai as Pe,aK as Re,ad as Dt,M as Ct}from"./D8zG9Ut8.js";import{d as Pt,V as Se,c as Rt,f as ze,g as zt,D as Nt,u as Ot}from"./CThvs1cs.js";import{_ as U}from"./DlAUqK2U.js";import{V as Ne}from"./QtiAwKG4.js";import{V as Vt}from"./CcxfNGfZ.js";import{T as Ue,V as Lt}from"./BRjp3Mpn.js";import{S as Mt}from"./DkCZQGJD.js";import{R as Bt,g as ge}from"./BvH6OxSR.js";const qt=ge`query scans($scanIds: [Int]) {
    scans(scanIds: $scanIds) {
        scans {
            id
            title
            progress
            riskRating
            agentGroup {
              id
              yamlSource
            }
            assets {
                __typename
                ... on OxoAndroidFileAssetType {
                  id
                  packageName
                  path
                }
                ... on OxoIOSFileAssetType {
                  id
                  bundleId
                  path
                }
                ... on OxoAndroidStoreAssetType {
                  id
                  packageName
                  applicationName
                }
                ... on OxoIOSStoreAssetType {
                  id
                  bundleId
                  applicationName
                }
                ... on OxoUrlsAssetType {
                  id
                  links {
                    url
                    method
                  }
                }
                ... on OxoNetworkAssetType {
                  id
                  networks {
                    host
                    mask
                  }
                }
                ... on OxoDomainNameAssetsType {
                  id
                  domainNames {
                    name
                  }
                }
            }
            createdTime
            kbVulnerabilities {
                highestRiskRating
                highestCvssV3Vector
                highestCvssV3BaseScore
                kb {
                    title
                    shortDescription
                    description
                    recommendation
                    references {
                        title
                        url
                    }
                }
                vulnerabilities {
                    vulnerabilities {
                        id
                        technicalDetail
                        riskRating
                        cvssV3Vector
                        detail {
                            title
                            shortDescription
                            description
                            recommendation
                            references {
                                title
                                url
                            }
                        }
                    }
                }
            }
        }
    }
}
`,Ht=ge`
query Vulnerabilities(
    $scanId: Int!,
    $kbDetailTitle: String,
    $vulnDetailTitles: [String!]!,
    $numberElements: Int,
    $page: Int
) {
    scan(scanId: $scanId) {
        id
        kbVulnerabilities(detailTitle: $kbDetailTitle) {
            highestRiskRating
            vulnerabilities(detailTitles: $vulnDetailTitles, numberElements: $numberElements, page: $page) {
                pageInfo {
                  count
                  numPages
                  hasNext
                  hasPrevious
                }
                vulnerabilities {
                    id
                    technicalDetail
                    riskRating
                    dna
                    detail {
                        title
                        shortDescription
                        description
                        recommendation
                        references {
                          title
                          url
                        }
                    }
                }
            }
        }
    }
}
`,Ut=ge`
query Vulnerabilities (
    $scanId: Int!,
    $vulnIds: [Int!]!,
) {
    scan(scanId: $scanId) {
        id
        vulnerabilities(vulnIds: $vulnIds) {
            vulnerabilities {
                id
                technicalDetail
                riskRating
                cvssV3Vector
                dna
                cvssV3BaseScore
                detail {
                    title
                    shortDescription
                    description
                    recommendation
                    references{
                        title
                        url
                    }
                }
            }
        }
    }
}
`,jt=ge`
query Vulnerabilities (
    $scanId: Int!,
    $page: Int!,
    $numberElements: Int!,
    $detailTitles: [String!]!
) {
    scan(scanId: $scanId) {
        id
        vulnerabilities(page: $page, numberElements: $numberElements, detailTitles: $detailTitles) {
            pageInfo {
              hasNext
              hasPrevious
              count
              numPages
            }
            vulnerabilities {
                id
                technicalDetail
                riskRating
                cvssV3Vector
                cvssV3BaseScore
            }
        }
    }
}
`;class ye{constructor(n){be(this,"requestHandler");be(this,"hasNext");this.requestHandler=new Bt(n),this.hasNext=!0}async getKBVulnerabilities(n,t){var d,s,r;const a=await this.requestHandler.post(n,{query:qt,variables:{scanIds:[t]}});return(((r=(s=(d=a==null?void 0:a.data)==null?void 0:d.data)==null?void 0:s.scans)==null?void 0:r.scans)||[])[0]||{}}async getVulnerabilitiesWithInfo({scanner:n,scanId:t,vulnDetailTitles:a,kbDetailTitle:d,page:s,numberElements:r}){var c,o,p,m,k,_,x,w,S;const i=await this.requestHandler.post(n,{query:Ht,variables:{scanId:t,vulnDetailTitles:(a||[]).filter(Boolean),page:s,numberElements:r,kbDetailTitle:d}});return this.hasNext=((_=(k=(m=(p=(o=(c=i==null?void 0:i.data)==null?void 0:c.data)==null?void 0:o.scan)==null?void 0:p.kbVulnerabilities[0])==null?void 0:m.vulnerabilities)==null?void 0:k.pageInfo)==null?void 0:_.hasNext)||!1,((S=(w=(x=i==null?void 0:i.data)==null?void 0:x.data)==null?void 0:w.scan)==null?void 0:S.kbVulnerabilities)||[]}async getVulnerabilityByIdWithInfo({scanner:n,scanId:t,vulnIds:a}){var s,r,i;const d=await this.requestHandler.post(n,{query:Ut,variables:{scanId:t,vulnIds:a}});return((i=(r=(s=d==null?void 0:d.data)==null?void 0:s.data)==null?void 0:r.scan)==null?void 0:i.vulnerabilities)||{vulnerabilities:[]}}async getVulnerabilitiesWithoutInfo({scanner:n,scanId:t,detailTitles:a,page:d,numberElements:s}){var i,c,o,p,m,k,_,x,w;const r=await this.requestHandler.post(n,{query:jt,variables:{scanId:t,detailTitles:a,page:d,numberElements:s}});return this.hasNext=((m=(p=(o=(c=(i=r==null?void 0:r.data)==null?void 0:i.data)==null?void 0:c.scan)==null?void 0:o.vulnerabilities)==null?void 0:p.pageInfo)==null?void 0:m.hasNext)||!1,((w=(x=(_=(k=r==null?void 0:r.data)==null?void 0:k.data)==null?void 0:_.scan)==null?void 0:x.vulnerabilities)==null?void 0:w.vulnerabilities)||[]}}var je=function(e){return function(n,t,a){return e(n,t,a)*a}},_e=function(e,n){if(e)throw Error("Invalid sort config: "+n)},Ge=function(e){var n=e||{},t=n.asc,a=n.desc,d=t?1:-1,s=t||a;_e(!s,"Expected `asc` or `desc` property"),_e(t&&a,"Ambiguous object with `asc` and `desc` config properties");var r=e.comparer&&je(e.comparer);return{order:d,sortBy:s,comparer:r}},Gt=function(e){return function n(t,a,d,s,r,i,c){var o,p;if(typeof t=="string")o=i[t],p=c[t];else if(typeof t=="function")o=t(i),p=t(c);else{var m=Ge(t);return n(m.sortBy,a,d,m.order,m.comparer||e,i,c)}var k=r(o,p,s);return(k===0||o==null&&p==null)&&a.length>d?n(a[d],a,d+1,s,r,i,c):k}};function We(e,n,t){if(e===void 0||e===!0)return function(s,r){return n(s,r,t)};if(typeof e=="string")return _e(e.includes("."),"String syntax not allowed for nested properties."),function(s,r){return n(s[e],r[e],t)};if(typeof e=="function")return function(s,r){return n(e(s),e(r),t)};if(Array.isArray(e)){var a=Gt(n);return function(s,r){return a(e[0],e,1,t,n,s,r)}}var d=Ge(e);return We(d.sortBy,d.comparer||n,d.order)}var ve=function(e,n,t,a){var d;return Array.isArray(n)?(Array.isArray(t)&&t.length<2&&(d=t,t=d[0]),n.sort(We(t,a,e))):n};function Ze(e){var n=je(e.comparer);return function(t){var a=Array.isArray(t)&&!e.inPlaceSorting?t.slice():t;return{asc:function(d){return ve(1,a,d,n)},desc:function(d){return ve(-1,a,d,n)},by:function(d){return ve(1,a,d,n)}}}}var Xe=function(e,n,t){return e==null?t:n==null?-t:typeof e!=typeof n?typeof e<typeof n?-1:1:e<n?-1:e>n?1:0},Wt=Ze({comparer:Xe});Ze({comparer:Xe,inPlaceSorting:!0});const Zt=new Error("request for lock canceled");var Xt=function(e,n,t,a){function d(s){return s instanceof t?s:new t(function(r){r(s)})}return new(t||(t=Promise))(function(s,r){function i(p){try{o(a.next(p))}catch(m){r(m)}}function c(p){try{o(a.throw(p))}catch(m){r(m)}}function o(p){p.done?s(p.value):d(p.value).then(i,c)}o((a=a.apply(e,n||[])).next())})};class Yt{constructor(n,t=Zt){this._value=n,this._cancelError=t,this._queue=[],this._weightedWaiters=[]}acquire(n=1,t=0){if(n<=0)throw new Error(`invalid weight ${n}: must be positive`);return new Promise((a,d)=>{const s={resolve:a,reject:d,weight:n,priority:t},r=Ye(this._queue,i=>t<=i.priority);r===-1&&n<=this._value?this._dispatchItem(s):this._queue.splice(r+1,0,s)})}runExclusive(n){return Xt(this,arguments,void 0,function*(t,a=1,d=0){const[s,r]=yield this.acquire(a,d);try{return yield t(s)}finally{r()}})}waitForUnlock(n=1,t=0){if(n<=0)throw new Error(`invalid weight ${n}: must be positive`);return this._couldLockImmediately(n,t)?Promise.resolve():new Promise(a=>{this._weightedWaiters[n-1]||(this._weightedWaiters[n-1]=[]),Kt(this._weightedWaiters[n-1],{resolve:a,priority:t})})}isLocked(){return this._value<=0}getValue(){return this._value}setValue(n){this._value=n,this._dispatchQueue()}release(n=1){if(n<=0)throw new Error(`invalid weight ${n}: must be positive`);this._value+=n,this._dispatchQueue()}cancel(){this._queue.forEach(n=>n.reject(this._cancelError)),this._queue=[]}_dispatchQueue(){for(this._drainUnlockWaiters();this._queue.length>0&&this._queue[0].weight<=this._value;)this._dispatchItem(this._queue.shift()),this._drainUnlockWaiters()}_dispatchItem(n){const t=this._value;this._value-=n.weight,n.resolve([t,this._newReleaser(n.weight)])}_newReleaser(n){let t=!1;return()=>{t||(t=!0,this.release(n))}}_drainUnlockWaiters(){if(this._queue.length===0)for(let n=this._value;n>0;n--){const t=this._weightedWaiters[n-1];t&&(t.forEach(a=>a.resolve()),this._weightedWaiters[n-1]=[])}else{const n=this._queue[0].priority;for(let t=this._value;t>0;t--){const a=this._weightedWaiters[t-1];if(!a)continue;const d=a.findIndex(s=>s.priority<=n);(d===-1?a:a.splice(0,d)).forEach(s=>s.resolve())}}}_couldLockImmediately(n,t){return(this._queue.length===0||this._queue[0].priority<t)&&n<=this._value}}function Kt(e,n){const t=Ye(e,a=>n.priority<=a.priority);e.splice(t+1,0,n)}function Ye(e,n){for(let t=e.length-1;t>=0;t--)if(n(e[t]))return t;return-1}var Qt=function(e,n,t,a){function d(s){return s instanceof t?s:new t(function(r){r(s)})}return new(t||(t=Promise))(function(s,r){function i(p){try{o(a.next(p))}catch(m){r(m)}}function c(p){try{o(a.throw(p))}catch(m){r(m)}}function o(p){p.done?s(p.value):d(p.value).then(i,c)}o((a=a.apply(e,n||[])).next())})};class Jt{constructor(n){this._semaphore=new Yt(1,n)}acquire(){return Qt(this,arguments,void 0,function*(n=0){const[,t]=yield this._semaphore.acquire(1,n);return t})}runExclusive(n,t=0){return this._semaphore.runExclusive(()=>n(),1,t)}isLocked(){return this._semaphore.isLocked()}waitForUnlock(n=0){return this._semaphore.waitForUnlock(1,n)}release(){this._semaphore.isLocked()&&this._semaphore.release()}cancel(){return this._semaphore.cancel()}}const en=H({name:"VulnerabilityTitle",components:{DfRisk:Pt},props:{title:{type:String,required:!0},riskRating:{type:String,default:null},loading:{type:Boolean,default:!1}}}),tn={class:"text-h5 mb-2",style:{"font-size":"2rem","font-weight":"400"}},nn={key:1},an={class:"d-flex align-center"};function sn(e,n,t,a,d,s){const r=N("DfRisk");return v(),T(Se,{class:"d-flex flex-column pa-3 vuln-details-page",variant:"outlined"},{default:D(()=>[R("p",tn,[e.loading===!0?(v(),T(Ne,{key:0,"max-width":"500",type:"text@1"})):(v(),I("span",nn,M(e.title),1))]),R("div",an,[e.loading===!0?(v(),T(Ne,{key:0,type:"button",style:{"flex-grow":"1"}})):(v(),T(r,{key:1,risk:e.riskRating,class:"mr-3"},null,8,["risk"]))])]),_:1})}const rn=U(en,[["render",sn]]),on=H({name:"MaterialCard",props:{avatar:{type:String,default:""},tile:{type:Boolean,default:!1},color:{type:String,default:"success"},hoverReveal:{type:Boolean,default:!1},icon:{type:String,default:void 0},image:{type:Boolean,default:!1},inline:{type:Boolean,default:!1},text:{type:String,default:""},title:{type:String,default:""}},computed:{classes(){return{"v-card--material--has-heading":this.hasHeading,"v-card--material--hover-reveal":this.hoverReveal}},hasHeading(){return!!(this.$slots.heading||this.title||this.icon)},hasAltHeading(){return!!(this.$slots.heading||this.title&&this.icon)}}}),ln={class:"d-flex grow flex-wrap"},un={key:2,class:"display-1 font-weight-bold"},dn={key:4,class:"headline font-weight-thin"},cn={key:2,class:"ml-6"},pn={key:4,class:"ml-4 d-flex"},gn={class:"font-weight-bold",style:{"font-size":"1.3rem"}},fn={class:"ml-3 mt-2",style:{"font-size":"0.9rem"}};function hn(e,n,t,a,d,s){return v(),T(Se,Z({variant:"outlined"},e.$attrs,{class:[e.classes,"v-card--material pa-3"],style:{overflow:"visible"}}),{default:D(()=>[R("div",ln,[e.avatar?(v(),T(ot,{key:0,size:"128",class:"mx-auto v-card--material__avatar elevation-12",tile:e.tile},{default:D(()=>[F(Me,{src:e.avatar},null,8,["src"])]),_:1},8,["tile"])):(v(),T(lt,{key:1,class:ut([{"pa-7":!e.$slots.image},"text-start v-card--material__heading mb-n6"]),color:e.color,"max-height":e.icon?75:void 0,width:e.inline||e.icon?"auto":"100%",dark:"",style:{"border-radius":"4px"}},{default:D(()=>[e.$slots.heading?j(e.$slots,"heading",{key:0}):e.$slots.image?j(e.$slots,"image",{key:1}):e.title&&!e.icon?(v(),I("p",un,M(e.title),1)):e.icon?(v(),T(oe,{key:3,size:"24"},{default:D(()=>[le(M(e.icon),1)]),_:1})):z("",!0),e.text?(v(),I("p",dn,M(e.text),1)):z("",!0)]),_:3},8,["class","color","max-height","width"])),e.$slots["after-heading"]?(v(),I("div",cn,[j(e.$slots,"after-heading")])):z("",!0),e.hoverReveal?(v(),T(Vt,{key:3,cols:"12",class:"text-center py-0 mt-n12"},{default:D(()=>[j(e.$slots,"reveal-actions")]),_:3})):e.icon&&e.title?(v(),I("div",pn,[R("div",gn,M(e.title),1),R("div",fn,[j(e.$slots,"title-extra-info")])])):z("",!0)]),j(e.$slots,"default"),e.$slots.actions?(v(),I(V,{key:0},[F(Be,{class:"mt-2"}),F(Rt,{class:"pb-0"},{default:D(()=>[j(e.$slots,"actions")]),_:3})],64)):z("",!0)]),_:3},16,["class"])}const mn=U(on,[["render",hn]]);var Ke={exports:{}};(function(e){var n=typeof window<"u"?window:typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope?self:{};/**
 * Prism: Lightweight, robust, elegant syntax highlighting
 *
 * @license MIT <https://opensource.org/licenses/MIT>
 * @author Lea Verou <https://lea.verou.me>
 * @namespace
 * @public
 */var t=function(a){var d=/(?:^|\s)lang(?:uage)?-([\w-]+)(?=\s|$)/i,s=0,r={},i={manual:a.Prism&&a.Prism.manual,disableWorkerMessageHandler:a.Prism&&a.Prism.disableWorkerMessageHandler,util:{encode:function l(u){return u instanceof c?new c(u.type,l(u.content),u.alias):Array.isArray(u)?u.map(l):u.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/\u00a0/g," ")},type:function(l){return Object.prototype.toString.call(l).slice(8,-1)},objId:function(l){return l.__id||Object.defineProperty(l,"__id",{value:++s}),l.__id},clone:function l(u,g){g=g||{};var f,h;switch(i.util.type(u)){case"Object":if(h=i.util.objId(u),g[h])return g[h];f={},g[h]=f;for(var b in u)u.hasOwnProperty(b)&&(f[b]=l(u[b],g));return f;case"Array":return h=i.util.objId(u),g[h]?g[h]:(f=[],g[h]=f,u.forEach(function($,y){f[y]=l($,g)}),f);default:return u}},getLanguage:function(l){for(;l;){var u=d.exec(l.className);if(u)return u[1].toLowerCase();l=l.parentElement}return"none"},setLanguage:function(l,u){l.className=l.className.replace(RegExp(d,"gi"),""),l.classList.add("language-"+u)},currentScript:function(){if(typeof document>"u")return null;if("currentScript"in document)return document.currentScript;try{throw new Error}catch(f){var l=(/at [^(\r\n]*\((.*):[^:]+:[^:]+\)$/i.exec(f.stack)||[])[1];if(l){var u=document.getElementsByTagName("script");for(var g in u)if(u[g].src==l)return u[g]}return null}},isActive:function(l,u,g){for(var f="no-"+u;l;){var h=l.classList;if(h.contains(u))return!0;if(h.contains(f))return!1;l=l.parentElement}return!!g}},languages:{plain:r,plaintext:r,text:r,txt:r,extend:function(l,u){var g=i.util.clone(i.languages[l]);for(var f in u)g[f]=u[f];return g},insertBefore:function(l,u,g,f){f=f||i.languages;var h=f[l],b={};for(var $ in h)if(h.hasOwnProperty($)){if($==u)for(var y in g)g.hasOwnProperty(y)&&(b[y]=g[y]);g.hasOwnProperty($)||(b[$]=h[$])}var E=f[l];return f[l]=b,i.languages.DFS(i.languages,function(C,B){B===E&&C!=l&&(this[C]=b)}),b},DFS:function l(u,g,f,h){h=h||{};var b=i.util.objId;for(var $ in u)if(u.hasOwnProperty($)){g.call(u,$,u[$],f||$);var y=u[$],E=i.util.type(y);E==="Object"&&!h[b(y)]?(h[b(y)]=!0,l(y,g,null,h)):E==="Array"&&!h[b(y)]&&(h[b(y)]=!0,l(y,g,$,h))}}},plugins:{},highlightAll:function(l,u){i.highlightAllUnder(document,l,u)},highlightAllUnder:function(l,u,g){var f={callback:g,container:l,selector:'code[class*="language-"], [class*="language-"] code, code[class*="lang-"], [class*="lang-"] code'};i.hooks.run("before-highlightall",f),f.elements=Array.prototype.slice.apply(f.container.querySelectorAll(f.selector)),i.hooks.run("before-all-elements-highlight",f);for(var h=0,b;b=f.elements[h++];)i.highlightElement(b,u===!0,f.callback)},highlightElement:function(l,u,g){var f=i.util.getLanguage(l),h=i.languages[f];i.util.setLanguage(l,f);var b=l.parentElement;b&&b.nodeName.toLowerCase()==="pre"&&i.util.setLanguage(b,f);var $=l.textContent,y={element:l,language:f,grammar:h,code:$};function E(B){y.highlightedCode=B,i.hooks.run("before-insert",y),y.element.innerHTML=y.highlightedCode,i.hooks.run("after-highlight",y),i.hooks.run("complete",y),g&&g.call(y.element)}if(i.hooks.run("before-sanity-check",y),b=y.element.parentElement,b&&b.nodeName.toLowerCase()==="pre"&&!b.hasAttribute("tabindex")&&b.setAttribute("tabindex","0"),!y.code){i.hooks.run("complete",y),g&&g.call(y.element);return}if(i.hooks.run("before-highlight",y),!y.grammar){E(i.util.encode(y.code));return}if(u&&a.Worker){var C=new Worker(i.filename);C.onmessage=function(B){E(B.data)},C.postMessage(JSON.stringify({language:y.language,code:y.code,immediateClose:!0}))}else E(i.highlight(y.code,y.grammar,y.language))},highlight:function(l,u,g){var f={code:l,grammar:u,language:g};if(i.hooks.run("before-tokenize",f),!f.grammar)throw new Error('The language "'+f.language+'" has no grammar.');return f.tokens=i.tokenize(f.code,f.grammar),i.hooks.run("after-tokenize",f),c.stringify(i.util.encode(f.tokens),f.language)},tokenize:function(l,u){var g=u.rest;if(g){for(var f in g)u[f]=g[f];delete u.rest}var h=new m;return k(h,h.head,l),p(l,h,u,h.head,0),x(h)},hooks:{all:{},add:function(l,u){var g=i.hooks.all;g[l]=g[l]||[],g[l].push(u)},run:function(l,u){var g=i.hooks.all[l];if(!(!g||!g.length))for(var f=0,h;h=g[f++];)h(u)}},Token:c};a.Prism=i;function c(l,u,g,f){this.type=l,this.content=u,this.alias=g,this.length=(f||"").length|0}c.stringify=function l(u,g){if(typeof u=="string")return u;if(Array.isArray(u)){var f="";return u.forEach(function(E){f+=l(E,g)}),f}var h={type:u.type,content:l(u.content,g),tag:"span",classes:["token",u.type],attributes:{},language:g},b=u.alias;b&&(Array.isArray(b)?Array.prototype.push.apply(h.classes,b):h.classes.push(b)),i.hooks.run("wrap",h);var $="";for(var y in h.attributes)$+=" "+y+'="'+(h.attributes[y]||"").replace(/"/g,"&quot;")+'"';return"<"+h.tag+' class="'+h.classes.join(" ")+'"'+$+">"+h.content+"</"+h.tag+">"};function o(l,u,g,f){l.lastIndex=u;var h=l.exec(g);if(h&&f&&h[1]){var b=h[1].length;h.index+=b,h[0]=h[0].slice(b)}return h}function p(l,u,g,f,h,b){for(var $ in g)if(!(!g.hasOwnProperty($)||!g[$])){var y=g[$];y=Array.isArray(y)?y:[y];for(var E=0;E<y.length;++E){if(b&&b.cause==$+","+E)return;var C=y[E],B=C.inside,ee=!!C.lookbehind,te=!!C.greedy,ne=C.alias;if(te&&!C.pattern.global){var nt=C.pattern.toString().match(/[imsuy]*$/)[0];C.pattern=RegExp(C.pattern.source,nt+"g")}for(var Ae=C.pattern||C,P=f.next,L=h;P!==u.tail&&!(b&&L>=b.reach);L+=P.value.length,P=P.next){var Y=P.value;if(u.length>l.length)return;if(!(Y instanceof c)){var ae=1,O;if(te){if(O=o(Ae,L,l,ee),!O||O.index>=l.length)break;var ie=O.index,at=O.index+O[0].length,q=L;for(q+=P.value.length;ie>=q;)P=P.next,q+=P.value.length;if(q-=P.value.length,L=q,P.value instanceof c)continue;for(var K=P;K!==u.tail&&(q<at||typeof K.value=="string");K=K.next)ae++,q+=K.value.length;ae--,Y=l.slice(L,q),O.index-=L}else if(O=o(Ae,0,Y,ee),!O)continue;var ie=O.index,se=O[0],fe=Y.slice(0,ie),Ee=Y.slice(ie+se.length),he=L+Y.length;b&&he>b.reach&&(b.reach=he);var re=P.prev;fe&&(re=k(u,re,fe),L+=fe.length),_(u,re,ae);var it=new c($,B?i.tokenize(se,B):se,ne,se);if(P=k(u,re,it),Ee&&k(u,P,Ee),ae>1){var me={cause:$+","+E,reach:he};p(l,u,g,P.prev,L,me),b&&me.reach>b.reach&&(b.reach=me.reach)}}}}}}function m(){var l={value:null,prev:null,next:null},u={value:null,prev:l,next:null};l.next=u,this.head=l,this.tail=u,this.length=0}function k(l,u,g){var f=u.next,h={value:g,prev:u,next:f};return u.next=h,f.prev=h,l.length++,h}function _(l,u,g){for(var f=u.next,h=0;h<g&&f!==l.tail;h++)f=f.next;u.next=f,f.prev=u,l.length-=h}function x(l){for(var u=[],g=l.head.next;g!==l.tail;)u.push(g.value),g=g.next;return u}if(!a.document)return a.addEventListener&&(i.disableWorkerMessageHandler||a.addEventListener("message",function(l){var u=JSON.parse(l.data),g=u.language,f=u.code,h=u.immediateClose;a.postMessage(i.highlight(f,i.languages[g],g)),h&&a.close()},!1)),i;var w=i.util.currentScript();w&&(i.filename=w.src,w.hasAttribute("data-manual")&&(i.manual=!0));function S(){i.manual||i.highlightAll()}if(!i.manual){var A=document.readyState;A==="loading"||A==="interactive"&&w&&w.defer?document.addEventListener("DOMContentLoaded",S):window.requestAnimationFrame?window.requestAnimationFrame(S):window.setTimeout(S,16)}return i}(n);e.exports&&(e.exports=t),typeof ze<"u"&&(ze.Prism=t),t.languages.markup={comment:{pattern:/<!--(?:(?!<!--)[\s\S])*?-->/,greedy:!0},prolog:{pattern:/<\?[\s\S]+?\?>/,greedy:!0},doctype:{pattern:/<!DOCTYPE(?:[^>"'[\]]|"[^"]*"|'[^']*')+(?:\[(?:[^<"'\]]|"[^"]*"|'[^']*'|<(?!!--)|<!--(?:[^-]|-(?!->))*-->)*\]\s*)?>/i,greedy:!0,inside:{"internal-subset":{pattern:/(^[^\[]*\[)[\s\S]+(?=\]>$)/,lookbehind:!0,greedy:!0,inside:null},string:{pattern:/"[^"]*"|'[^']*'/,greedy:!0},punctuation:/^<!|>$|[[\]]/,"doctype-tag":/^DOCTYPE/i,name:/[^\s<>'"]+/}},cdata:{pattern:/<!\[CDATA\[[\s\S]*?\]\]>/i,greedy:!0},tag:{pattern:/<\/?(?!\d)[^\s>\/=$<%]+(?:\s(?:\s*[^\s>\/=]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?=[\s/>])))+)?\s*\/?>/,greedy:!0,inside:{tag:{pattern:/^<\/?[^\s>\/]+/,inside:{punctuation:/^<\/?/,namespace:/^[^\s>\/:]+:/}},"special-attr":[],"attr-value":{pattern:/=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+)/,inside:{punctuation:[{pattern:/^=/,alias:"attr-equals"},{pattern:/^(\s*)["']|["']$/,lookbehind:!0}]}},punctuation:/\/?>/,"attr-name":{pattern:/[^\s>\/]+/,inside:{namespace:/^[^\s>\/:]+:/}}}},entity:[{pattern:/&[\da-z]{1,8};/i,alias:"named-entity"},/&#x?[\da-f]{1,8};/i]},t.languages.markup.tag.inside["attr-value"].inside.entity=t.languages.markup.entity,t.languages.markup.doctype.inside["internal-subset"].inside=t.languages.markup,t.hooks.add("wrap",function(a){a.type==="entity"&&(a.attributes.title=a.content.replace(/&amp;/,"&"))}),Object.defineProperty(t.languages.markup.tag,"addInlined",{value:function(d,s){var r={};r["language-"+s]={pattern:/(^<!\[CDATA\[)[\s\S]+?(?=\]\]>$)/i,lookbehind:!0,inside:t.languages[s]},r.cdata=/^<!\[CDATA\[|\]\]>$/i;var i={"included-cdata":{pattern:/<!\[CDATA\[[\s\S]*?\]\]>/i,inside:r}};i["language-"+s]={pattern:/[\s\S]+/,inside:t.languages[s]};var c={};c[d]={pattern:RegExp(/(<__[^>]*>)(?:<!\[CDATA\[(?:[^\]]|\](?!\]>))*\]\]>|(?!<!\[CDATA\[)[\s\S])*?(?=<\/__>)/.source.replace(/__/g,function(){return d}),"i"),lookbehind:!0,greedy:!0,inside:i},t.languages.insertBefore("markup","cdata",c)}}),Object.defineProperty(t.languages.markup.tag,"addAttribute",{value:function(a,d){t.languages.markup.tag.inside["special-attr"].push({pattern:RegExp(/(^|["'\s])/.source+"(?:"+a+")"+/\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))/.source,"i"),lookbehind:!0,inside:{"attr-name":/^[^\s=]+/,"attr-value":{pattern:/=[\s\S]+/,inside:{value:{pattern:/(^=\s*(["']|(?!["'])))\S[\s\S]*(?=\2$)/,lookbehind:!0,alias:[d,"language-"+d],inside:t.languages[d]},punctuation:[{pattern:/^=/,alias:"attr-equals"},/"|'/]}}}})}}),t.languages.html=t.languages.markup,t.languages.mathml=t.languages.markup,t.languages.svg=t.languages.markup,t.languages.xml=t.languages.extend("markup",{}),t.languages.ssml=t.languages.xml,t.languages.atom=t.languages.xml,t.languages.rss=t.languages.xml,function(a){var d=/(?:"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n])*')/;a.languages.css={comment:/\/\*[\s\S]*?\*\//,atrule:{pattern:RegExp("@[\\w-](?:"+/[^;{\s"']|\s+(?!\s)/.source+"|"+d.source+")*?"+/(?:;|(?=\s*\{))/.source),inside:{rule:/^@[\w-]+/,"selector-function-argument":{pattern:/(\bselector\s*\(\s*(?![\s)]))(?:[^()\s]|\s+(?![\s)])|\((?:[^()]|\([^()]*\))*\))+(?=\s*\))/,lookbehind:!0,alias:"selector"},keyword:{pattern:/(^|[^\w-])(?:and|not|only|or)(?![\w-])/,lookbehind:!0}}},url:{pattern:RegExp("\\burl\\((?:"+d.source+"|"+/(?:[^\\\r\n()"']|\\[\s\S])*/.source+")\\)","i"),greedy:!0,inside:{function:/^url/i,punctuation:/^\(|\)$/,string:{pattern:RegExp("^"+d.source+"$"),alias:"url"}}},selector:{pattern:RegExp(`(^|[{}\\s])[^{}\\s](?:[^{};"'\\s]|\\s+(?![\\s{])|`+d.source+")*(?=\\s*\\{)"),lookbehind:!0},string:{pattern:d,greedy:!0},property:{pattern:/(^|[^-\w\xA0-\uFFFF])(?!\s)[-_a-z\xA0-\uFFFF](?:(?!\s)[-\w\xA0-\uFFFF])*(?=\s*:)/i,lookbehind:!0},important:/!important\b/i,function:{pattern:/(^|[^-a-z0-9])[-a-z0-9]+(?=\()/i,lookbehind:!0},punctuation:/[(){};:,]/},a.languages.css.atrule.inside.rest=a.languages.css;var s=a.languages.markup;s&&(s.tag.addInlined("style","css"),s.tag.addAttribute("style","css"))}(t),t.languages.clike={comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},"class-name":{pattern:/(\b(?:class|extends|implements|instanceof|interface|new|trait)\s+|\bcatch\s+\()[\w.\\]+/i,lookbehind:!0,inside:{punctuation:/[.\\]/}},keyword:/\b(?:break|catch|continue|do|else|finally|for|function|if|in|instanceof|new|null|return|throw|try|while)\b/,boolean:/\b(?:false|true)\b/,function:/\b\w+(?=\()/,number:/\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,operator:/[<>]=?|[!=]=?=?|--?|\+\+?|&&?|\|\|?|[?*/~^%]/,punctuation:/[{}[\];(),.:]/},t.languages.javascript=t.languages.extend("clike",{"class-name":[t.languages.clike["class-name"],{pattern:/(^|[^$\w\xA0-\uFFFF])(?!\s)[_$A-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\.(?:constructor|prototype))/,lookbehind:!0}],keyword:[{pattern:/((?:^|\})\s*)catch\b/,lookbehind:!0},{pattern:/(^|[^.]|\.\.\.\s*)\b(?:as|assert(?=\s*\{)|async(?=\s*(?:function\b|\(|[$\w\xA0-\uFFFF]|$))|await|break|case|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally(?=\s*(?:\{|$))|for|from(?=\s*(?:['"]|$))|function|(?:get|set)(?=\s*(?:[#\[$\w\xA0-\uFFFF]|$))|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)\b/,lookbehind:!0}],function:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*(?:\.\s*(?:apply|bind|call)\s*)?\()/,number:{pattern:RegExp(/(^|[^\w$])/.source+"(?:"+(/NaN|Infinity/.source+"|"+/0[bB][01]+(?:_[01]+)*n?/.source+"|"+/0[oO][0-7]+(?:_[0-7]+)*n?/.source+"|"+/0[xX][\dA-Fa-f]+(?:_[\dA-Fa-f]+)*n?/.source+"|"+/\d+(?:_\d+)*n/.source+"|"+/(?:\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\.\d+(?:_\d+)*)(?:[Ee][+-]?\d+(?:_\d+)*)?/.source)+")"+/(?![\w$])/.source),lookbehind:!0},operator:/--|\+\+|\*\*=?|=>|&&=?|\|\|=?|[!=]==|<<=?|>>>?=?|[-+*/%&|^!=<>]=?|\.{3}|\?\?=?|\?\.?|[~:]/}),t.languages.javascript["class-name"][0].pattern=/(\b(?:class|extends|implements|instanceof|interface|new)\s+)[\w.\\]+/,t.languages.insertBefore("javascript","keyword",{regex:{pattern:RegExp(/((?:^|[^$\w\xA0-\uFFFF."'\])\s]|\b(?:return|yield))\s*)/.source+/\//.source+"(?:"+/(?:\[(?:[^\]\\\r\n]|\\.)*\]|\\.|[^/\\\[\r\n])+\/[dgimyus]{0,7}/.source+"|"+/(?:\[(?:[^[\]\\\r\n]|\\.|\[(?:[^[\]\\\r\n]|\\.|\[(?:[^[\]\\\r\n]|\\.)*\])*\])*\]|\\.|[^/\\\[\r\n])+\/[dgimyus]{0,7}v[dgimyus]{0,7}/.source+")"+/(?=(?:\s|\/\*(?:[^*]|\*(?!\/))*\*\/)*(?:$|[\r\n,.;:})\]]|\/\/))/.source),lookbehind:!0,greedy:!0,inside:{"regex-source":{pattern:/^(\/)[\s\S]+(?=\/[a-z]*$)/,lookbehind:!0,alias:"language-regex",inside:t.languages.regex},"regex-delimiter":/^\/|\/$/,"regex-flags":/^[a-z]+$/}},"function-variable":{pattern:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*[=:]\s*(?:async\s*)?(?:\bfunction\b|(?:\((?:[^()]|\([^()]*\))*\)|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/,alias:"function"},parameter:[{pattern:/(function(?:\s+(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)?\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\))/,lookbehind:!0,inside:t.languages.javascript},{pattern:/(^|[^$\w\xA0-\uFFFF])(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*=>)/i,lookbehind:!0,inside:t.languages.javascript},{pattern:/(\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*=>)/,lookbehind:!0,inside:t.languages.javascript},{pattern:/((?:\b|\s|^)(?!(?:as|async|await|break|case|catch|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally|for|from|function|get|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)(?![$\w\xA0-\uFFFF]))(?:(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*)\(\s*|\]\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*\{)/,lookbehind:!0,inside:t.languages.javascript}],constant:/\b[A-Z](?:[A-Z_]|\dx?)*\b/}),t.languages.insertBefore("javascript","string",{hashbang:{pattern:/^#!.*/,greedy:!0,alias:"comment"},"template-string":{pattern:/`(?:\\[\s\S]|\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}|(?!\$\{)[^\\`])*`/,greedy:!0,inside:{"template-punctuation":{pattern:/^`|`$/,alias:"string"},interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,lookbehind:!0,inside:{"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"},rest:t.languages.javascript}},string:/[\s\S]+/}},"string-property":{pattern:/((?:^|[,{])[ \t]*)(["'])(?:\\(?:\r\n|[\s\S])|(?!\2)[^\\\r\n])*\2(?=\s*:)/m,lookbehind:!0,greedy:!0,alias:"property"}}),t.languages.insertBefore("javascript","operator",{"literal-property":{pattern:/((?:^|[,{])[ \t]*)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/m,lookbehind:!0,alias:"property"}}),t.languages.markup&&(t.languages.markup.tag.addInlined("script","javascript"),t.languages.markup.tag.addAttribute(/on(?:abort|blur|change|click|composition(?:end|start|update)|dblclick|error|focus(?:in|out)?|key(?:down|up)|load|mouse(?:down|enter|leave|move|out|over|up)|reset|resize|scroll|select|slotchange|submit|unload|wheel)/.source,"javascript")),t.languages.js=t.languages.javascript,function(){if(typeof t>"u"||typeof document>"u")return;Element.prototype.matches||(Element.prototype.matches=Element.prototype.msMatchesSelector||Element.prototype.webkitMatchesSelector);var a="Loading…",d=function(w,S){return"✖ Error "+w+" while fetching file: "+S},s="✖ Error: File does not exist or is empty",r={js:"javascript",py:"python",rb:"ruby",ps1:"powershell",psm1:"powershell",sh:"bash",bat:"batch",h:"c",tex:"latex"},i="data-src-status",c="loading",o="loaded",p="failed",m="pre[data-src]:not(["+i+'="'+o+'"]):not(['+i+'="'+c+'"])';function k(w,S,A){var l=new XMLHttpRequest;l.open("GET",w,!0),l.onreadystatechange=function(){l.readyState==4&&(l.status<400&&l.responseText?S(l.responseText):l.status>=400?A(d(l.status,l.statusText)):A(s))},l.send(null)}function _(w){var S=/^\s*(\d+)\s*(?:(,)\s*(?:(\d+)\s*)?)?$/.exec(w||"");if(S){var A=Number(S[1]),l=S[2],u=S[3];return l?u?[A,Number(u)]:[A,void 0]:[A,A]}}t.hooks.add("before-highlightall",function(w){w.selector+=", "+m}),t.hooks.add("before-sanity-check",function(w){var S=w.element;if(S.matches(m)){w.code="",S.setAttribute(i,c);var A=S.appendChild(document.createElement("CODE"));A.textContent=a;var l=S.getAttribute("data-src"),u=w.language;if(u==="none"){var g=(/\.(\w+)$/.exec(l)||[,"none"])[1];u=r[g]||g}t.util.setLanguage(A,u),t.util.setLanguage(S,u);var f=t.plugins.autoloader;f&&f.loadLanguages(u),k(l,function(h){S.setAttribute(i,o);var b=_(S.getAttribute("data-range"));if(b){var $=h.split(/\r\n?|\n/g),y=b[0],E=b[1]==null?$.length:b[1];y<0&&(y+=$.length),y=Math.max(0,Math.min(y-1,$.length)),E<0&&(E+=$.length),E=Math.max(0,Math.min(E,$.length)),h=$.slice(y,E).join(`
`),S.hasAttribute("data-start")||S.setAttribute("data-start",String(y+1))}A.textContent=h,t.highlightElement(A)},function(h){S.setAttribute(i,p),A.textContent=h})}}),t.plugins.fileHighlight={highlight:function(S){for(var A=(S||document).querySelectorAll(m),l=0,u;u=A[l++];)t.highlightElement(u)}};var x=!1;t.fileHighlight=function(){x||(console.warn("Prism.fileHighlight is deprecated. Use `Prism.plugins.fileHighlight.highlight` instead."),x=!0),t.plugins.fileHighlight.highlight.apply(this,arguments)}}()})(Ke);var bn=Ke.exports;const Oe=zt(bn);(function(e){var n="\\b(?:BASH|BASHOPTS|BASH_ALIASES|BASH_ARGC|BASH_ARGV|BASH_CMDS|BASH_COMPLETION_COMPAT_DIR|BASH_LINENO|BASH_REMATCH|BASH_SOURCE|BASH_VERSINFO|BASH_VERSION|COLORTERM|COLUMNS|COMP_WORDBREAKS|DBUS_SESSION_BUS_ADDRESS|DEFAULTS_PATH|DESKTOP_SESSION|DIRSTACK|DISPLAY|EUID|GDMSESSION|GDM_LANG|GNOME_KEYRING_CONTROL|GNOME_KEYRING_PID|GPG_AGENT_INFO|GROUPS|HISTCONTROL|HISTFILE|HISTFILESIZE|HISTSIZE|HOME|HOSTNAME|HOSTTYPE|IFS|INSTANCE|JOB|LANG|LANGUAGE|LC_ADDRESS|LC_ALL|LC_IDENTIFICATION|LC_MEASUREMENT|LC_MONETARY|LC_NAME|LC_NUMERIC|LC_PAPER|LC_TELEPHONE|LC_TIME|LESSCLOSE|LESSOPEN|LINES|LOGNAME|LS_COLORS|MACHTYPE|MAILCHECK|MANDATORY_PATH|NO_AT_BRIDGE|OLDPWD|OPTERR|OPTIND|ORBIT_SOCKETDIR|OSTYPE|PAPERSIZE|PATH|PIPESTATUS|PPID|PS1|PS2|PS3|PS4|PWD|RANDOM|REPLY|SECONDS|SELINUX_INIT|SESSION|SESSIONTYPE|SESSION_MANAGER|SHELL|SHELLOPTS|SHLVL|SSH_AUTH_SOCK|TERM|UID|UPSTART_EVENTS|UPSTART_INSTANCE|UPSTART_JOB|UPSTART_SESSION|USER|WINDOWID|XAUTHORITY|XDG_CONFIG_DIRS|XDG_CURRENT_DESKTOP|XDG_DATA_DIRS|XDG_GREETER_DATA_DIR|XDG_MENU_PREFIX|XDG_RUNTIME_DIR|XDG_SEAT|XDG_SEAT_PATH|XDG_SESSION_DESKTOP|XDG_SESSION_ID|XDG_SESSION_PATH|XDG_SESSION_TYPE|XDG_VTNR|XMODIFIERS)\\b",t={pattern:/(^(["']?)\w+\2)[ \t]+\S.*/,lookbehind:!0,alias:"punctuation",inside:null},a={bash:t,environment:{pattern:RegExp("\\$"+n),alias:"constant"},variable:[{pattern:/\$?\(\([\s\S]+?\)\)/,greedy:!0,inside:{variable:[{pattern:/(^\$\(\([\s\S]+)\)\)/,lookbehind:!0},/^\$\(\(/],number:/\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,operator:/--|\+\+|\*\*=?|<<=?|>>=?|&&|\|\||[=!+\-*/%<>^&|]=?|[?~:]/,punctuation:/\(\(?|\)\)?|,|;/}},{pattern:/\$\((?:\([^)]+\)|[^()])+\)|`[^`]+`/,greedy:!0,inside:{variable:/^\$\(|^`|\)$|`$/}},{pattern:/\$\{[^}]+\}/,greedy:!0,inside:{operator:/:[-=?+]?|[!\/]|##?|%%?|\^\^?|,,?/,punctuation:/[\[\]]/,environment:{pattern:RegExp("(\\{)"+n),lookbehind:!0,alias:"constant"}}},/\$(?:\w+|[#?*!@$])/],entity:/\\(?:[abceEfnrtv\\"]|O?[0-7]{1,3}|U[0-9a-fA-F]{8}|u[0-9a-fA-F]{4}|x[0-9a-fA-F]{1,2})/};e.languages.bash={shebang:{pattern:/^#!\s*\/.*/,alias:"important"},comment:{pattern:/(^|[^"{\\$])#.*/,lookbehind:!0},"function-name":[{pattern:/(\bfunction\s+)[\w-]+(?=(?:\s*\(?:\s*\))?\s*\{)/,lookbehind:!0,alias:"function"},{pattern:/\b[\w-]+(?=\s*\(\s*\)\s*\{)/,alias:"function"}],"for-or-select":{pattern:/(\b(?:for|select)\s+)\w+(?=\s+in\s)/,alias:"variable",lookbehind:!0},"assign-left":{pattern:/(^|[\s;|&]|[<>]\()\w+(?:\.\w+)*(?=\+?=)/,inside:{environment:{pattern:RegExp("(^|[\\s;|&]|[<>]\\()"+n),lookbehind:!0,alias:"constant"}},alias:"variable",lookbehind:!0},parameter:{pattern:/(^|\s)-{1,2}(?:\w+:[+-]?)?\w+(?:\.\w+)*(?=[=\s]|$)/,alias:"variable",lookbehind:!0},string:[{pattern:/((?:^|[^<])<<-?\s*)(\w+)\s[\s\S]*?(?:\r?\n|\r)\2/,lookbehind:!0,greedy:!0,inside:a},{pattern:/((?:^|[^<])<<-?\s*)(["'])(\w+)\2\s[\s\S]*?(?:\r?\n|\r)\3/,lookbehind:!0,greedy:!0,inside:{bash:t}},{pattern:/(^|[^\\](?:\\\\)*)"(?:\\[\s\S]|\$\([^)]+\)|\$(?!\()|`[^`]+`|[^"\\`$])*"/,lookbehind:!0,greedy:!0,inside:a},{pattern:/(^|[^$\\])'[^']*'/,lookbehind:!0,greedy:!0},{pattern:/\$'(?:[^'\\]|\\[\s\S])*'/,greedy:!0,inside:{entity:a.entity}}],environment:{pattern:RegExp("\\$?"+n),alias:"constant"},variable:a.variable,function:{pattern:/(^|[\s;|&]|[<>]\()(?:add|apropos|apt|apt-cache|apt-get|aptitude|aspell|automysqlbackup|awk|basename|bash|bc|bconsole|bg|bzip2|cal|cargo|cat|cfdisk|chgrp|chkconfig|chmod|chown|chroot|cksum|clear|cmp|column|comm|composer|cp|cron|crontab|csplit|curl|cut|date|dc|dd|ddrescue|debootstrap|df|diff|diff3|dig|dir|dircolors|dirname|dirs|dmesg|docker|docker-compose|du|egrep|eject|env|ethtool|expand|expect|expr|fdformat|fdisk|fg|fgrep|file|find|fmt|fold|format|free|fsck|ftp|fuser|gawk|git|gparted|grep|groupadd|groupdel|groupmod|groups|grub-mkconfig|gzip|halt|head|hg|history|host|hostname|htop|iconv|id|ifconfig|ifdown|ifup|import|install|ip|java|jobs|join|kill|killall|less|link|ln|locate|logname|logrotate|look|lpc|lpr|lprint|lprintd|lprintq|lprm|ls|lsof|lynx|make|man|mc|mdadm|mkconfig|mkdir|mke2fs|mkfifo|mkfs|mkisofs|mknod|mkswap|mmv|more|most|mount|mtools|mtr|mutt|mv|nano|nc|netstat|nice|nl|node|nohup|notify-send|npm|nslookup|op|open|parted|passwd|paste|pathchk|ping|pkill|pnpm|podman|podman-compose|popd|pr|printcap|printenv|ps|pushd|pv|quota|quotacheck|quotactl|ram|rar|rcp|reboot|remsync|rename|renice|rev|rm|rmdir|rpm|rsync|scp|screen|sdiff|sed|sendmail|seq|service|sftp|sh|shellcheck|shuf|shutdown|sleep|slocate|sort|split|ssh|stat|strace|su|sudo|sum|suspend|swapon|sync|sysctl|tac|tail|tar|tee|time|timeout|top|touch|tr|traceroute|tsort|tty|umount|uname|unexpand|uniq|units|unrar|unshar|unzip|update-grub|uptime|useradd|userdel|usermod|users|uudecode|uuencode|v|vcpkg|vdir|vi|vim|virsh|vmstat|wait|watch|wc|wget|whereis|which|who|whoami|write|xargs|xdg-open|yarn|yes|zenity|zip|zsh|zypper)(?=$|[)\s;|&])/,lookbehind:!0},keyword:{pattern:/(^|[\s;|&]|[<>]\()(?:case|do|done|elif|else|esac|fi|for|function|if|in|select|then|until|while)(?=$|[)\s;|&])/,lookbehind:!0},builtin:{pattern:/(^|[\s;|&]|[<>]\()(?:\.|:|alias|bind|break|builtin|caller|cd|command|continue|declare|echo|enable|eval|exec|exit|export|getopts|hash|help|let|local|logout|mapfile|printf|pwd|read|readarray|readonly|return|set|shift|shopt|source|test|times|trap|type|typeset|ulimit|umask|unalias|unset)(?=$|[)\s;|&])/,lookbehind:!0,alias:"class-name"},boolean:{pattern:/(^|[\s;|&]|[<>]\()(?:false|true)(?=$|[)\s;|&])/,lookbehind:!0},"file-descriptor":{pattern:/\B&\d\b/,alias:"important"},operator:{pattern:/\d?<>|>\||\+=|=[=~]?|!=?|<<[<-]?|[&\d]?>>|\d[<>]&?|[<>][&=]?|&[>&]?|\|[&|]?/,inside:{"file-descriptor":{pattern:/^\d/,alias:"important"}}},punctuation:/\$?\(\(?|\)\)?|\.\.|[{}[\];\\]/,number:{pattern:/(^|\s)(?:[1-9]\d*|0)(?:[.,]\d+)?\b/,lookbehind:!0}},t.inside=e.languages.bash;for(var d=["comment","function-name","for-or-select","assign-left","parameter","string","environment","function","keyword","builtin","boolean","file-descriptor","operator","punctuation","number"],s=a.variable[1].inside,r=0;r<d.length;r++)s[d[r]]=e.languages.bash[d[r]];e.languages.sh=e.languages.bash,e.languages.shell=e.languages.bash})(Prism);Prism.languages.javascript=Prism.languages.extend("clike",{"class-name":[Prism.languages.clike["class-name"],{pattern:/(^|[^$\w\xA0-\uFFFF])(?!\s)[_$A-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\.(?:constructor|prototype))/,lookbehind:!0}],keyword:[{pattern:/((?:^|\})\s*)catch\b/,lookbehind:!0},{pattern:/(^|[^.]|\.\.\.\s*)\b(?:as|assert(?=\s*\{)|async(?=\s*(?:function\b|\(|[$\w\xA0-\uFFFF]|$))|await|break|case|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally(?=\s*(?:\{|$))|for|from(?=\s*(?:['"]|$))|function|(?:get|set)(?=\s*(?:[#\[$\w\xA0-\uFFFF]|$))|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)\b/,lookbehind:!0}],function:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*(?:\.\s*(?:apply|bind|call)\s*)?\()/,number:{pattern:RegExp("(^|[^\\w$])(?:NaN|Infinity|0[bB][01]+(?:_[01]+)*n?|0[oO][0-7]+(?:_[0-7]+)*n?|0[xX][\\dA-Fa-f]+(?:_[\\dA-Fa-f]+)*n?|\\d+(?:_\\d+)*n|(?:\\d+(?:_\\d+)*(?:\\.(?:\\d+(?:_\\d+)*)?)?|\\.\\d+(?:_\\d+)*)(?:[Ee][+-]?\\d+(?:_\\d+)*)?)(?![\\w$])"),lookbehind:!0},operator:/--|\+\+|\*\*=?|=>|&&=?|\|\|=?|[!=]==|<<=?|>>>?=?|[-+*/%&|^!=<>]=?|\.{3}|\?\?=?|\?\.?|[~:]/}),Prism.languages.javascript["class-name"][0].pattern=/(\b(?:class|extends|implements|instanceof|interface|new)\s+)[\w.\\]+/,Prism.languages.insertBefore("javascript","keyword",{regex:{pattern:RegExp(`((?:^|[^$\\w\\xA0-\\uFFFF."'\\])\\s]|\\b(?:return|yield))\\s*)/(?:(?:\\[(?:[^\\]\\\\\r
]|\\\\.)*\\]|\\\\.|[^/\\\\\\[\r
])+/[dgimyus]{0,7}|(?:\\[(?:[^[\\]\\\\\r
]|\\\\.|\\[(?:[^[\\]\\\\\r
]|\\\\.|\\[(?:[^[\\]\\\\\r
]|\\\\.)*\\])*\\])*\\]|\\\\.|[^/\\\\\\[\r
])+/[dgimyus]{0,7}v[dgimyus]{0,7})(?=(?:\\s|/\\*(?:[^*]|\\*(?!/))*\\*/)*(?:$|[\r
,.;:})\\]]|//))`),lookbehind:!0,greedy:!0,inside:{"regex-source":{pattern:/^(\/)[\s\S]+(?=\/[a-z]*$)/,lookbehind:!0,alias:"language-regex",inside:Prism.languages.regex},"regex-delimiter":/^\/|\/$/,"regex-flags":/^[a-z]+$/}},"function-variable":{pattern:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*[=:]\s*(?:async\s*)?(?:\bfunction\b|(?:\((?:[^()]|\([^()]*\))*\)|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/,alias:"function"},parameter:[{pattern:/(function(?:\s+(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)?\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\))/,lookbehind:!0,inside:Prism.languages.javascript},{pattern:/(^|[^$\w\xA0-\uFFFF])(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*=>)/i,lookbehind:!0,inside:Prism.languages.javascript},{pattern:/(\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*=>)/,lookbehind:!0,inside:Prism.languages.javascript},{pattern:/((?:\b|\s|^)(?!(?:as|async|await|break|case|catch|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally|for|from|function|get|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)(?![$\w\xA0-\uFFFF]))(?:(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*)\(\s*|\]\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*\{)/,lookbehind:!0,inside:Prism.languages.javascript}],constant:/\b[A-Z](?:[A-Z_]|\dx?)*\b/}),Prism.languages.insertBefore("javascript","string",{hashbang:{pattern:/^#!.*/,greedy:!0,alias:"comment"},"template-string":{pattern:/`(?:\\[\s\S]|\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}|(?!\$\{)[^\\`])*`/,greedy:!0,inside:{"template-punctuation":{pattern:/^`|`$/,alias:"string"},interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,lookbehind:!0,inside:{"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"},rest:Prism.languages.javascript}},string:/[\s\S]+/}},"string-property":{pattern:/((?:^|[,{])[ \t]*)(["'])(?:\\(?:\r\n|[\s\S])|(?!\2)[^\\\r\n])*\2(?=\s*:)/m,lookbehind:!0,greedy:!0,alias:"property"}}),Prism.languages.insertBefore("javascript","operator",{"literal-property":{pattern:/((?:^|[,{])[ \t]*)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/m,lookbehind:!0,alias:"property"}}),Prism.languages.markup&&(Prism.languages.markup.tag.addInlined("script","javascript"),Prism.languages.markup.tag.addAttribute("on(?:abort|blur|change|click|composition(?:end|start|update)|dblclick|error|focus(?:in|out)?|key(?:down|up)|load|mouse(?:down|enter|leave|move|out|over|up)|reset|resize|scroll|select|slotchange|submit|unload|wheel)","javascript")),Prism.languages.js=Prism.languages.javascript;Prism.languages.json={property:{pattern:/(^|[^\\])"(?:\\.|[^\\"\r\n])*"(?=\s*:)/,lookbehind:!0,greedy:!0},string:{pattern:/(^|[^\\])"(?:\\.|[^\\"\r\n])*"(?!\s*:)/,lookbehind:!0,greedy:!0},comment:{pattern:/\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,greedy:!0},number:/-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,punctuation:/[{}[\],]/,operator:/:/,boolean:/\b(?:false|true)\b/,null:{pattern:/\bnull\b/,alias:"keyword"}},Prism.languages.webmanifest=Prism.languages.json;(function(e){e.languages.kotlin=e.languages.extend("clike",{keyword:{pattern:/(^|[^.])\b(?:abstract|actual|annotation|as|break|by|catch|class|companion|const|constructor|continue|crossinline|data|do|dynamic|else|enum|expect|external|final|finally|for|fun|get|if|import|in|infix|init|inline|inner|interface|internal|is|lateinit|noinline|null|object|open|operator|out|override|package|private|protected|public|reified|return|sealed|set|super|suspend|tailrec|this|throw|to|try|typealias|val|var|vararg|when|where|while)\b/,lookbehind:!0},function:[{pattern:/(?:`[^\r\n`]+`|\b\w+)(?=\s*\()/,greedy:!0},{pattern:/(\.)(?:`[^\r\n`]+`|\w+)(?=\s*\{)/,lookbehind:!0,greedy:!0}],number:/\b(?:0[xX][\da-fA-F]+(?:_[\da-fA-F]+)*|0[bB][01]+(?:_[01]+)*|\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?(?:[eE][+-]?\d+(?:_\d+)*)?[fFL]?)\b/,operator:/\+[+=]?|-[-=>]?|==?=?|!(?:!|==?)?|[\/*%<>]=?|[?:]:?|\.\.|&&|\|\||\b(?:and|inv|or|shl|shr|ushr|xor)\b/}),delete e.languages.kotlin["class-name"];var n={"interpolation-punctuation":{pattern:/^\$\{?|\}$/,alias:"punctuation"},expression:{pattern:/[\s\S]+/,inside:e.languages.kotlin}};e.languages.insertBefore("kotlin","string",{"string-literal":[{pattern:/"""(?:[^$]|\$(?:(?!\{)|\{[^{}]*\}))*?"""/,alias:"multiline",inside:{interpolation:{pattern:/\$(?:[a-z_]\w*|\{[^{}]*\})/i,inside:n},string:/[\s\S]+/}},{pattern:/"(?:[^"\\\r\n$]|\\.|\$(?:(?!\{)|\{[^{}]*\}))*"/,alias:"singleline",inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$(?:[a-z_]\w*|\{[^{}]*\})/i,lookbehind:!0,inside:n},string:/[\s\S]+/}}],char:{pattern:/'(?:[^'\\\r\n]|\\(?:.|u[a-fA-F0-9]{0,4}))'/,greedy:!0}}),delete e.languages.kotlin.string,e.languages.insertBefore("kotlin","keyword",{annotation:{pattern:/\B@(?:\w+:)?(?:[A-Z]\w*|\[[^\]]+\])/,alias:"builtin"}}),e.languages.insertBefore("kotlin","function",{label:{pattern:/\b\w+@|@\w+\b/,alias:"symbol"}}),e.languages.kt=e.languages.kotlin,e.languages.kts=e.languages.kotlin})(Prism);(function(e){function n(c){return c=c.replace(/<inner>/g,function(){return`(?:\\\\.|[^\\\\
\r]|(?:
|\r
?)(?![\r
]))`}),RegExp("((?:^|[^\\\\])(?:\\\\{2})*)(?:"+c+")")}var t="(?:\\\\.|``(?:[^`\r\n]|`(?!`))+``|`[^`\r\n]+`|[^\\\\|\r\n`])+",a=`\\|?__(?:\\|__)+\\|?(?:(?:
|\r
?)|(?![^]))`.replace(/__/g,function(){return t}),d=`\\|?[ 	]*:?-{3,}:?[ 	]*(?:\\|[ 	]*:?-{3,}:?[ 	]*)+\\|?(?:
|\r
?)`;e.languages.markdown=e.languages.extend("markup",{}),e.languages.insertBefore("markdown","prolog",{"front-matter-block":{pattern:/(^(?:\s*[\r\n])?)---(?!.)[\s\S]*?[\r\n]---(?!.)/,lookbehind:!0,greedy:!0,inside:{punctuation:/^---|---$/,"front-matter":{pattern:/\S+(?:\s+\S+)*/,alias:["yaml","language-yaml"],inside:e.languages.yaml}}},blockquote:{pattern:/^>(?:[\t ]*>)*/m,alias:"punctuation"},table:{pattern:RegExp("^"+a+d+"(?:"+a+")*","m"),inside:{"table-data-rows":{pattern:RegExp("^("+a+d+")(?:"+a+")*$"),lookbehind:!0,inside:{"table-data":{pattern:RegExp(t),inside:e.languages.markdown},punctuation:/\|/}},"table-line":{pattern:RegExp("^("+a+")"+d+"$"),lookbehind:!0,inside:{punctuation:/\||:?-{3,}:?/}},"table-header-row":{pattern:RegExp("^"+a+"$"),inside:{"table-header":{pattern:RegExp(t),alias:"important",inside:e.languages.markdown},punctuation:/\|/}}}},code:[{pattern:/((?:^|\n)[ \t]*\n|(?:^|\r\n?)[ \t]*\r\n?)(?: {4}|\t).+(?:(?:\n|\r\n?)(?: {4}|\t).+)*/,lookbehind:!0,alias:"keyword"},{pattern:/^```[\s\S]*?^```$/m,greedy:!0,inside:{"code-block":{pattern:/^(```.*(?:\n|\r\n?))[\s\S]+?(?=(?:\n|\r\n?)^```$)/m,lookbehind:!0},"code-language":{pattern:/^(```).+/,lookbehind:!0},punctuation:/```/}}],title:[{pattern:/\S.*(?:\n|\r\n?)(?:==+|--+)(?=[ \t]*$)/m,alias:"important",inside:{punctuation:/==+$|--+$/}},{pattern:/(^\s*)#.+/m,lookbehind:!0,alias:"important",inside:{punctuation:/^#+|#+$/}}],hr:{pattern:/(^\s*)([*-])(?:[\t ]*\2){2,}(?=\s*$)/m,lookbehind:!0,alias:"punctuation"},list:{pattern:/(^\s*)(?:[*+-]|\d+\.)(?=[\t ].)/m,lookbehind:!0,alias:"punctuation"},"url-reference":{pattern:/!?\[[^\]]+\]:[\t ]+(?:\S+|<(?:\\.|[^>\\])+>)(?:[\t ]+(?:"(?:\\.|[^"\\])*"|'(?:\\.|[^'\\])*'|\((?:\\.|[^)\\])*\)))?/,inside:{variable:{pattern:/^(!?\[)[^\]]+/,lookbehind:!0},string:/(?:"(?:\\.|[^"\\])*"|'(?:\\.|[^'\\])*'|\((?:\\.|[^)\\])*\))$/,punctuation:/^[\[\]!:]|[<>]/},alias:"url"},bold:{pattern:n("\\b__(?:(?!_)<inner>|_(?:(?!_)<inner>)+_)+__\\b|\\*\\*(?:(?!\\*)<inner>|\\*(?:(?!\\*)<inner>)+\\*)+\\*\\*"),lookbehind:!0,greedy:!0,inside:{content:{pattern:/(^..)[\s\S]+(?=..$)/,lookbehind:!0,inside:{}},punctuation:/\*\*|__/}},italic:{pattern:n("\\b_(?:(?!_)<inner>|__(?:(?!_)<inner>)+__)+_\\b|\\*(?:(?!\\*)<inner>|\\*\\*(?:(?!\\*)<inner>)+\\*\\*)+\\*"),lookbehind:!0,greedy:!0,inside:{content:{pattern:/(^.)[\s\S]+(?=.$)/,lookbehind:!0,inside:{}},punctuation:/[*_]/}},strike:{pattern:n("(~~?)(?:(?!~)<inner>)+\\2"),lookbehind:!0,greedy:!0,inside:{content:{pattern:/(^~~?)[\s\S]+(?=\1$)/,lookbehind:!0,inside:{}},punctuation:/~~?/}},"code-snippet":{pattern:/(^|[^\\`])(?:``[^`\r\n]+(?:`[^`\r\n]+)*``(?!`)|`[^`\r\n]+`(?!`))/,lookbehind:!0,greedy:!0,alias:["code","keyword"]},url:{pattern:n('!?\\[(?:(?!\\])<inner>)+\\](?:\\([^\\s)]+(?:[	 ]+"(?:\\\\.|[^"\\\\])*")?\\)|[ 	]?\\[(?:(?!\\])<inner>)+\\])'),lookbehind:!0,greedy:!0,inside:{operator:/^!/,content:{pattern:/(^\[)[^\]]+(?=\])/,lookbehind:!0,inside:{}},variable:{pattern:/(^\][ \t]?\[)[^\]]+(?=\]$)/,lookbehind:!0},url:{pattern:/(^\]\()[^\s)]+/,lookbehind:!0},string:{pattern:/(^[ \t]+)"(?:\\.|[^"\\])*"(?=\)$)/,lookbehind:!0}}}}),["url","bold","italic","strike"].forEach(function(c){["url","bold","italic","strike","code-snippet"].forEach(function(o){c!==o&&(e.languages.markdown[c].inside.content.inside[o]=e.languages.markdown[o])})}),e.hooks.add("after-tokenize",function(c){c.language!=="markdown"&&c.language!=="md"||function o(p){if(p&&typeof p!="string")for(var m=0,k=p.length;m<k;m++){var _=p[m];if(_.type==="code"){var x=_.content[1],w=_.content[3];if(x&&w&&x.type==="code-language"&&w.type==="code-block"&&typeof x.content=="string"){var S=x.content.replace(/\b#/g,"sharp").replace(/\b\+\+/g,"pp"),A="language-"+(S=(/[a-z][\w-]*/i.exec(S)||[""])[0].toLowerCase());w.alias?typeof w.alias=="string"?w.alias=[w.alias,A]:w.alias.push(A):w.alias=[A]}}else o(_.content)}}(c.tokens)}),e.hooks.add("wrap",function(c){if(c.type==="code-block"){for(var o="",p=0,m=c.classes.length;p<m;p++){var k=c.classes[p],_=/language-(.+)/.exec(k);if(_){o=_[1];break}}var x=e.languages[o];if(x)c.content=e.highlight(c.content.replace(s,"").replace(/&(\w{1,8}|#x?[\da-f]{1,8});/gi,function(S,A){var l;return(A=A.toLowerCase())[0]==="#"?(l=A[1]==="x"?parseInt(A.slice(2),16):Number(A.slice(1)),i(l)):r[A]||S}),x,o);else if(o&&o!=="none"&&e.plugins.autoloader){var w="md-"+new Date().valueOf()+"-"+Math.floor(1e16*Math.random());c.attributes.id=w,e.plugins.autoloader.loadLanguages(o,function(){var S=document.getElementById(w);S&&(S.innerHTML=e.highlight(S.textContent,e.languages[o],o))})}}});var s=RegExp(e.languages.markup.tag.pattern.source,"gi"),r={amp:"&",lt:"<",gt:">",quot:'"'},i=String.fromCodePoint||String.fromCharCode;e.languages.md=e.languages.markdown})(Prism);(function(e){function n(t,a){return"___"+t.toUpperCase()+a+"___"}Object.defineProperties(e.languages["markup-templating"]={},{buildPlaceholders:{value:function(t,a,d,s){if(t.language===a){var r=t.tokenStack=[];t.code=t.code.replace(d,function(i){if(typeof s=="function"&&!s(i))return i;for(var c,o=r.length;t.code.indexOf(c=n(a,o))!==-1;)++o;return r[o]=i,c}),t.grammar=e.languages.markup}}},tokenizePlaceholders:{value:function(t,a){if(t.language===a&&t.tokenStack){t.grammar=e.languages[a];var d=0,s=Object.keys(t.tokenStack);(function r(i){for(var c=0;c<i.length&&!(d>=s.length);c++){var o=i[c];if(typeof o=="string"||o.content&&typeof o.content=="string"){var p=s[d],m=t.tokenStack[p],k=typeof o=="string"?o:o.content,_=n(a,p),x=k.indexOf(_);if(x>-1){++d;var w=k.substring(0,x),S=new e.Token(a,e.tokenize(m,t.grammar),"language-"+a,m),A=k.substring(x+_.length),l=[];w&&l.push.apply(l,r([w])),l.push(S),A&&l.push.apply(l,r([A])),typeof o=="string"?i.splice.apply(i,[c,1].concat(l)):o.content=l}}else o.content&&r(o.content)}return i})(t.tokens)}}}})})(Prism);(function(e){var n=/\/\*[\s\S]*?\*\/|\/\/.*|#(?!\[).*/,t=[{pattern:/\b(?:false|true)\b/i,alias:"boolean"},{pattern:/(::\s*)\b[a-z_]\w*\b(?!\s*\()/i,greedy:!0,lookbehind:!0},{pattern:/(\b(?:case|const)\s+)\b[a-z_]\w*(?=\s*[;=])/i,greedy:!0,lookbehind:!0},/\b(?:null)\b/i,/\b[A-Z_][A-Z0-9_]*\b(?!\s*\()/],a=/\b0b[01]+(?:_[01]+)*\b|\b0o[0-7]+(?:_[0-7]+)*\b|\b0x[\da-f]+(?:_[\da-f]+)*\b|(?:\b\d+(?:_\d+)*\.?(?:\d+(?:_\d+)*)?|\B\.\d+)(?:e[+-]?\d+)?/i,d=/<?=>|\?\?=?|\.{3}|\??->|[!=]=?=?|::|\*\*=?|--|\+\+|&&|\|\||<<|>>|[?~]|[/^|%*&<>.+-]=?/,s=/[{}\[\](),:;]/;e.languages.php={delimiter:{pattern:/\?>$|^<\?(?:php(?=\s)|=)?/i,alias:"important"},comment:n,variable:/\$+(?:\w+\b|(?=\{))/,package:{pattern:/(namespace\s+|use\s+(?:function\s+)?)(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,lookbehind:!0,inside:{punctuation:/\\/}},"class-name-definition":{pattern:/(\b(?:class|enum|interface|trait)\s+)\b[a-z_]\w*(?!\\)\b/i,lookbehind:!0,alias:"class-name"},"function-definition":{pattern:/(\bfunction\s+)[a-z_]\w*(?=\s*\()/i,lookbehind:!0,alias:"function"},keyword:[{pattern:/(\(\s*)\b(?:array|bool|boolean|float|int|integer|object|string)\b(?=\s*\))/i,alias:"type-casting",greedy:!0,lookbehind:!0},{pattern:/([(,?]\s*)\b(?:array(?!\s*\()|bool|callable|(?:false|null)(?=\s*\|)|float|int|iterable|mixed|object|self|static|string)\b(?=\s*\$)/i,alias:"type-hint",greedy:!0,lookbehind:!0},{pattern:/(\)\s*:\s*(?:\?\s*)?)\b(?:array(?!\s*\()|bool|callable|(?:false|null)(?=\s*\|)|float|int|iterable|mixed|never|object|self|static|string|void)\b/i,alias:"return-type",greedy:!0,lookbehind:!0},{pattern:/\b(?:array(?!\s*\()|bool|float|int|iterable|mixed|object|string|void)\b/i,alias:"type-declaration",greedy:!0},{pattern:/(\|\s*)(?:false|null)\b|\b(?:false|null)(?=\s*\|)/i,alias:"type-declaration",greedy:!0,lookbehind:!0},{pattern:/\b(?:parent|self|static)(?=\s*::)/i,alias:"static-context",greedy:!0},{pattern:/(\byield\s+)from\b/i,lookbehind:!0},/\bclass\b/i,{pattern:/((?:^|[^\s>:]|(?:^|[^-])>|(?:^|[^:]):)\s*)\b(?:abstract|and|array|as|break|callable|case|catch|clone|const|continue|declare|default|die|do|echo|else|elseif|empty|enddeclare|endfor|endforeach|endif|endswitch|endwhile|enum|eval|exit|extends|final|finally|fn|for|foreach|function|global|goto|if|implements|include|include_once|instanceof|insteadof|interface|isset|list|match|namespace|never|new|or|parent|print|private|protected|public|readonly|require|require_once|return|self|static|switch|throw|trait|try|unset|use|var|while|xor|yield|__halt_compiler)\b/i,lookbehind:!0}],"argument-name":{pattern:/([(,]\s*)\b[a-z_]\w*(?=\s*:(?!:))/i,lookbehind:!0},"class-name":[{pattern:/(\b(?:extends|implements|instanceof|new(?!\s+self|\s+static))\s+|\bcatch\s*\()\b[a-z_]\w*(?!\\)\b/i,greedy:!0,lookbehind:!0},{pattern:/(\|\s*)\b[a-z_]\w*(?!\\)\b/i,greedy:!0,lookbehind:!0},{pattern:/\b[a-z_]\w*(?!\\)\b(?=\s*\|)/i,greedy:!0},{pattern:/(\|\s*)(?:\\?\b[a-z_]\w*)+\b/i,alias:"class-name-fully-qualified",greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}},{pattern:/(?:\\?\b[a-z_]\w*)+\b(?=\s*\|)/i,alias:"class-name-fully-qualified",greedy:!0,inside:{punctuation:/\\/}},{pattern:/(\b(?:extends|implements|instanceof|new(?!\s+self\b|\s+static\b))\s+|\bcatch\s*\()(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,alias:"class-name-fully-qualified",greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}},{pattern:/\b[a-z_]\w*(?=\s*\$)/i,alias:"type-declaration",greedy:!0},{pattern:/(?:\\?\b[a-z_]\w*)+(?=\s*\$)/i,alias:["class-name-fully-qualified","type-declaration"],greedy:!0,inside:{punctuation:/\\/}},{pattern:/\b[a-z_]\w*(?=\s*::)/i,alias:"static-context",greedy:!0},{pattern:/(?:\\?\b[a-z_]\w*)+(?=\s*::)/i,alias:["class-name-fully-qualified","static-context"],greedy:!0,inside:{punctuation:/\\/}},{pattern:/([(,?]\s*)[a-z_]\w*(?=\s*\$)/i,alias:"type-hint",greedy:!0,lookbehind:!0},{pattern:/([(,?]\s*)(?:\\?\b[a-z_]\w*)+(?=\s*\$)/i,alias:["class-name-fully-qualified","type-hint"],greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}},{pattern:/(\)\s*:\s*(?:\?\s*)?)\b[a-z_]\w*(?!\\)\b/i,alias:"return-type",greedy:!0,lookbehind:!0},{pattern:/(\)\s*:\s*(?:\?\s*)?)(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,alias:["class-name-fully-qualified","return-type"],greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}}],constant:t,function:{pattern:/(^|[^\\\w])\\?[a-z_](?:[\w\\]*\w)?(?=\s*\()/i,lookbehind:!0,inside:{punctuation:/\\/}},property:{pattern:/(->\s*)\w+/,lookbehind:!0},number:a,operator:d,punctuation:s};var r={pattern:/\{\$(?:\{(?:\{[^{}]+\}|[^{}]+)\}|[^{}])+\}|(^|[^\\{])\$+(?:\w+(?:\[[^\r\n\[\]]+\]|->\w+)?)/,lookbehind:!0,inside:e.languages.php},i=[{pattern:/<<<'([^']+)'[\r\n](?:.*[\r\n])*?\1;/,alias:"nowdoc-string",greedy:!0,inside:{delimiter:{pattern:/^<<<'[^']+'|[a-z_]\w*;$/i,alias:"symbol",inside:{punctuation:/^<<<'?|[';]$/}}}},{pattern:/<<<(?:"([^"]+)"[\r\n](?:.*[\r\n])*?\1;|([a-z_]\w*)[\r\n](?:.*[\r\n])*?\2;)/i,alias:"heredoc-string",greedy:!0,inside:{delimiter:{pattern:/^<<<(?:"[^"]+"|[a-z_]\w*)|[a-z_]\w*;$/i,alias:"symbol",inside:{punctuation:/^<<<"?|[";]$/}},interpolation:r}},{pattern:/`(?:\\[\s\S]|[^\\`])*`/,alias:"backtick-quoted-string",greedy:!0},{pattern:/'(?:\\[\s\S]|[^\\'])*'/,alias:"single-quoted-string",greedy:!0},{pattern:/"(?:\\[\s\S]|[^\\"])*"/,alias:"double-quoted-string",greedy:!0,inside:{interpolation:r}}];e.languages.insertBefore("php","variable",{string:i,attribute:{pattern:/#\[(?:[^"'\/#]|\/(?![*/])|\/\/.*$|#(?!\[).*$|\/\*(?:[^*]|\*(?!\/))*\*\/|"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*')+\](?=\s*[a-z$#])/im,greedy:!0,inside:{"attribute-content":{pattern:/^(#\[)[\s\S]+(?=\]$)/,lookbehind:!0,inside:{comment:n,string:i,"attribute-class-name":[{pattern:/([^:]|^)\b[a-z_]\w*(?!\\)\b/i,alias:"class-name",greedy:!0,lookbehind:!0},{pattern:/([^:]|^)(?:\\?\b[a-z_]\w*)+/i,alias:["class-name","class-name-fully-qualified"],greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}}],constant:t,number:a,operator:d,punctuation:s}},delimiter:{pattern:/^#\[|\]$/,alias:"punctuation"}}}}),e.hooks.add("before-tokenize",function(c){/<\?/.test(c.code)&&e.languages["markup-templating"].buildPlaceholders(c,"php",/<\?(?:[^"'/#]|\/(?![*/])|("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|(?:\/\/|#(?!\[))(?:[^?\n\r]|\?(?!>))*(?=$|\?>|[\r\n])|#\[|\/\*(?:[^*]|\*(?!\/))*(?:\*\/|$))*?(?:\?>|$)/g)}),e.hooks.add("after-tokenize",function(c){e.languages["markup-templating"].tokenizePlaceholders(c,"php")})})(Prism);(function(e){var n=/\b(?:abstract|assert|boolean|break|byte|case|catch|char|class|const|continue|default|do|double|else|enum|exports|extends|final|finally|float|for|goto|if|implements|import|instanceof|int|interface|long|module|native|new|non-sealed|null|open|opens|package|permits|private|protected|provides|public|record(?!\s*[(){}[\]<>=%~.:,;?+\-*/&|^])|requires|return|sealed|short|static|strictfp|super|switch|synchronized|this|throw|throws|to|transient|transitive|try|uses|var|void|volatile|while|with|yield)\b/,t="(?:[a-z]\\w*\\s*\\.\\s*)*(?:[A-Z]\\w*\\s*\\.\\s*)*",a={pattern:RegExp("(^|[^\\w.])"+t+"[A-Z](?:[\\d_A-Z]*[a-z]\\w*)?\\b"),lookbehind:!0,inside:{namespace:{pattern:/^[a-z]\w*(?:\s*\.\s*[a-z]\w*)*(?:\s*\.)?/,inside:{punctuation:/\./}},punctuation:/\./}};e.languages.java=e.languages.extend("clike",{string:{pattern:/(^|[^\\])"(?:\\.|[^"\\\r\n])*"/,lookbehind:!0,greedy:!0},"class-name":[a,{pattern:RegExp("(^|[^\\w.])"+t+"[A-Z]\\w*(?=\\s+\\w+\\s*[;,=()]|\\s*(?:\\[[\\s,]*\\]\\s*)?::\\s*new\\b)"),lookbehind:!0,inside:a.inside},{pattern:RegExp("(\\b(?:class|enum|extends|implements|instanceof|interface|new|record|throws)\\s+)"+t+"[A-Z]\\w*\\b"),lookbehind:!0,inside:a.inside}],keyword:n,function:[e.languages.clike.function,{pattern:/(::\s*)[a-z_]\w*/,lookbehind:!0}],number:/\b0b[01][01_]*L?\b|\b0x(?:\.[\da-f_p+-]+|[\da-f_]+(?:\.[\da-f_p+-]+)?)\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?\d[\d_]*)?[dfl]?/i,operator:{pattern:/(^|[^.])(?:<<=?|>>>?=?|->|--|\+\+|&&|\|\||::|[?:~]|[-+*/%&|^!=<>]=?)/m,lookbehind:!0},constant:/\b[A-Z][A-Z_\d]+\b/}),e.languages.insertBefore("java","string",{"triple-quoted-string":{pattern:/"""[ \t]*[\r\n](?:(?:"|"")?(?:\\.|[^"\\]))*"""/,greedy:!0,alias:"string"},char:{pattern:/'(?:\\.|[^'\\\r\n]){1,6}'/,greedy:!0}}),e.languages.insertBefore("java","class-name",{annotation:{pattern:/(^|[^.])@\w+(?:\s*\.\s*\w+)*/,lookbehind:!0,alias:"punctuation"},generics:{pattern:/<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&))*>)*>)*>)*>/,inside:{"class-name":a,keyword:n,punctuation:/[<>(),.:]/,operator:/[?&|]/}},import:[{pattern:RegExp("(\\bimport\\s+)"+t+"(?:[A-Z]\\w*|\\*)(?=\\s*;)"),lookbehind:!0,inside:{namespace:a.inside.namespace,punctuation:/\./,operator:/\*/,"class-name":/\w+/}},{pattern:RegExp("(\\bimport\\s+static\\s+)"+t+"(?:\\w+|\\*)(?=\\s*;)"),lookbehind:!0,alias:"static",inside:{namespace:a.inside.namespace,static:/\b\w+$/,punctuation:/\./,operator:/\*/,"class-name":/\w+/}}],namespace:{pattern:RegExp("(\\b(?:exports|import(?:\\s+static)?|module|open|opens|package|provides|requires|to|transitive|uses|with)\\s+)(?!<keyword>)[a-z]\\w*(?:\\.[a-z]\\w*)*\\.?".replace(/<keyword>/g,function(){return n.source})),lookbehind:!0,inside:{punctuation:/\./}}})})(Prism);Prism.languages.python={comment:{pattern:/(^|[^\\])#.*/,lookbehind:!0,greedy:!0},"string-interpolation":{pattern:/(?:f|fr|rf)(?:("""|''')[\s\S]*?\1|("|')(?:\\.|(?!\2)[^\\\r\n])*\2)/i,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^{])(?:\{\{)*)\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}])+\})+\})+\}/,lookbehind:!0,inside:{"format-spec":{pattern:/(:)[^:(){}]+(?=\}$)/,lookbehind:!0},"conversion-option":{pattern:/![sra](?=[:}]$)/,alias:"punctuation"},rest:null}},string:/[\s\S]+/}},"triple-quoted-string":{pattern:/(?:[rub]|br|rb)?("""|''')[\s\S]*?\1/i,greedy:!0,alias:"string"},string:{pattern:/(?:[rub]|br|rb)?("|')(?:\\.|(?!\1)[^\\\r\n])*\1/i,greedy:!0},function:{pattern:/((?:^|\s)def[ \t]+)[a-zA-Z_]\w*(?=\s*\()/g,lookbehind:!0},"class-name":{pattern:/(\bclass\s+)\w+/i,lookbehind:!0},decorator:{pattern:/(^[\t ]*)@\w+(?:\.\w+)*/m,lookbehind:!0,alias:["annotation","punctuation"],inside:{punctuation:/\./}},keyword:/\b(?:_(?=\s*:)|and|as|assert|async|await|break|case|class|continue|def|del|elif|else|except|exec|finally|for|from|global|if|import|in|is|lambda|match|nonlocal|not|or|pass|print|raise|return|try|while|with|yield)\b/,builtin:/\b(?:__import__|abs|all|any|apply|ascii|basestring|bin|bool|buffer|bytearray|bytes|callable|chr|classmethod|cmp|coerce|compile|complex|delattr|dict|dir|divmod|enumerate|eval|execfile|file|filter|float|format|frozenset|getattr|globals|hasattr|hash|help|hex|id|input|int|intern|isinstance|issubclass|iter|len|list|locals|long|map|max|memoryview|min|next|object|oct|open|ord|pow|property|range|raw_input|reduce|reload|repr|reversed|round|set|setattr|slice|sorted|staticmethod|str|sum|super|tuple|type|unichr|unicode|vars|xrange|zip)\b/,boolean:/\b(?:False|None|True)\b/,number:/\b0(?:b(?:_?[01])+|o(?:_?[0-7])+|x(?:_?[a-f0-9])+)\b|(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)(?:e[+-]?\d+(?:_\d+)*)?j?(?!\w)/i,operator:/[-+%=]=?|!=|:=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,punctuation:/[{}[\];(),.:]/},Prism.languages.python["string-interpolation"].inside.interpolation.inside.rest=Prism.languages.python,Prism.languages.py=Prism.languages.python;Prism.languages.swift={comment:{pattern:/(^|[^\\:])(?:\/\/.*|\/\*(?:[^/*]|\/(?!\*)|\*(?!\/)|\/\*(?:[^*]|\*(?!\/))*\*\/)*\*\/)/,lookbehind:!0,greedy:!0},"string-literal":[{pattern:RegExp(`(^|[^"#])(?:"(?:\\\\(?:\\((?:[^()]|\\([^()]*\\))*\\)|\r
|[^(])|[^\\\\\r
"])*"|"""(?:\\\\(?:\\((?:[^()]|\\([^()]*\\))*\\)|[^(])|[^\\\\"]|"(?!""))*""")(?!["#])`),lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/(\\\()(?:[^()]|\([^()]*\))*(?=\))/,lookbehind:!0,inside:null},"interpolation-punctuation":{pattern:/^\)|\\\($/,alias:"punctuation"},punctuation:/\\(?=[\r\n])/,string:/[\s\S]+/}},{pattern:RegExp(`(^|[^"#])(#+)(?:"(?:\\\\(?:#+\\((?:[^()]|\\([^()]*\\))*\\)|\r
|[^#])|[^\\\\\r
])*?"|"""(?:\\\\(?:#+\\((?:[^()]|\\([^()]*\\))*\\)|[^#])|[^\\\\])*?""")\\2`),lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/(\\#+\()(?:[^()]|\([^()]*\))*(?=\))/,lookbehind:!0,inside:null},"interpolation-punctuation":{pattern:/^\)|\\#+\($/,alias:"punctuation"},string:/[\s\S]+/}}],directive:{pattern:RegExp("#(?:(?:elseif|if)\\b(?:[ 	]*(?:![ 	]*)?(?:\\b\\w+\\b(?:[ 	]*\\((?:[^()]|\\([^()]*\\))*\\))?|\\((?:[^()]|\\([^()]*\\))*\\))(?:[ 	]*(?:&&|\\|\\|))?)+|(?:else|endif)\\b)"),alias:"property",inside:{"directive-name":/^#\w+/,boolean:/\b(?:false|true)\b/,number:/\b\d+(?:\.\d+)*\b/,operator:/!|&&|\|\||[<>]=?/,punctuation:/[(),]/}},literal:{pattern:/#(?:colorLiteral|column|dsohandle|file(?:ID|Literal|Path)?|function|imageLiteral|line)\b/,alias:"constant"},"other-directive":{pattern:/#\w+\b/,alias:"property"},attribute:{pattern:/@\w+/,alias:"atrule"},"function-definition":{pattern:/(\bfunc\s+)\w+/,lookbehind:!0,alias:"function"},label:{pattern:/\b(break|continue)\s+\w+|\b[a-zA-Z_]\w*(?=\s*:\s*(?:for|repeat|while)\b)/,lookbehind:!0,alias:"important"},keyword:/\b(?:Any|Protocol|Self|Type|actor|as|assignment|associatedtype|associativity|async|await|break|case|catch|class|continue|convenience|default|defer|deinit|didSet|do|dynamic|else|enum|extension|fallthrough|fileprivate|final|for|func|get|guard|higherThan|if|import|in|indirect|infix|init|inout|internal|is|isolated|lazy|left|let|lowerThan|mutating|none|nonisolated|nonmutating|open|operator|optional|override|postfix|precedencegroup|prefix|private|protocol|public|repeat|required|rethrows|return|right|safe|self|set|some|static|struct|subscript|super|switch|throw|throws|try|typealias|unowned|unsafe|var|weak|where|while|willSet)\b/,boolean:/\b(?:false|true)\b/,nil:{pattern:/\bnil\b/,alias:"constant"},"short-argument":/\$\d+\b/,omit:{pattern:/\b_\b/,alias:"keyword"},number:/\b(?:[\d_]+(?:\.[\de_]+)?|0x[a-f0-9_]+(?:\.[a-f0-9p_]+)?|0b[01_]+|0o[0-7_]+)\b/i,"class-name":/\b[A-Z](?:[A-Z_\d]*[a-z]\w*)?\b/,function:/\b[a-z_]\w*(?=\s*\()/i,constant:/\b(?:[A-Z_]{2,}|k[A-Z][A-Za-z_]+)\b/,operator:/[-+*/%=!<>&|^~?]+|\.[.\-+*/%=!<>&|^~?]+/,punctuation:/[{}[\]();,.:\\]/},Prism.languages.swift["string-literal"].forEach(function(e){e.inside.interpolation.inside=Prism.languages.swift});(function(e){var n=[/\b(?:async|sync|yield)\*/,/\b(?:abstract|assert|async|await|break|case|catch|class|const|continue|covariant|default|deferred|do|dynamic|else|enum|export|extends|extension|external|factory|final|finally|for|get|hide|if|implements|import|in|interface|library|mixin|new|null|on|operator|part|rethrow|return|set|show|static|super|switch|sync|this|throw|try|typedef|var|void|while|with|yield)\b/],t="(^|[^\\w.])(?:[a-z]\\w*\\s*\\.\\s*)*(?:[A-Z]\\w*\\s*\\.\\s*)*",a={pattern:RegExp(t+"[A-Z](?:[\\d_A-Z]*[a-z]\\w*)?\\b"),lookbehind:!0,inside:{namespace:{pattern:/^[a-z]\w*(?:\s*\.\s*[a-z]\w*)*(?:\s*\.)?/,inside:{punctuation:/\./}}}};e.languages.dart=e.languages.extend("clike",{"class-name":[a,{pattern:RegExp(t+"[A-Z]\\w*(?=\\s+\\w+\\s*[;,=()])"),lookbehind:!0,inside:a.inside}],keyword:n,operator:/\bis!|\b(?:as|is)\b|\+\+|--|&&|\|\||<<=?|>>=?|~(?:\/=?)?|[+\-*\/%&^|=!<>]=?|\?/}),e.languages.insertBefore("dart","string",{"string-literal":{pattern:/r?(?:("""|''')[\s\S]*?\1|(["'])(?:\\.|(?!\2)[^\\\r\n])*\2(?!\2))/,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$(?:\w+|\{(?:[^{}]|\{[^{}]*\})*\})/,lookbehind:!0,inside:{punctuation:/^\$\{?|\}$/,expression:{pattern:/[\s\S]+/,inside:e.languages.dart}}},string:/[\s\S]+/}},string:void 0}),e.languages.insertBefore("dart","class-name",{metadata:{pattern:/@\w+/,alias:"function"}}),e.languages.insertBefore("dart","class-name",{generics:{pattern:/<(?:[\w\s,.&?]|<(?:[\w\s,.&?]|<(?:[\w\s,.&?]|<[\w\s,.&?]*>)*>)*>)*>/,inside:{"class-name":a,keyword:n,punctuation:/[<>(),.:]/,operator:/[?&|]/}}})})(Prism);Prism.languages.c=Prism.languages.extend("clike",{comment:{pattern:/\/\/(?:[^\r\n\\]|\\(?:\r\n?|\n|(?![\r\n])))*|\/\*[\s\S]*?(?:\*\/|$)/,greedy:!0},string:{pattern:/"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,greedy:!0},"class-name":{pattern:/(\b(?:enum|struct)\s+(?:__attribute__\s*\(\([\s\S]*?\)\)\s*)?)\w+|\b[a-z]\w*_t\b/,lookbehind:!0},keyword:/\b(?:_Alignas|_Alignof|_Atomic|_Bool|_Complex|_Generic|_Imaginary|_Noreturn|_Static_assert|_Thread_local|__attribute__|asm|auto|break|case|char|const|continue|default|do|double|else|enum|extern|float|for|goto|if|inline|int|long|register|return|short|signed|sizeof|static|struct|switch|typedef|typeof|union|unsigned|void|volatile|while)\b/,function:/\b[a-z_]\w*(?=\s*\()/i,number:/(?:\b0x(?:[\da-f]+(?:\.[\da-f]*)?|\.[\da-f]+)(?:p[+-]?\d+)?|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[ful]{0,4}/i,operator:/>>=?|<<=?|->|([-+&|:])\1|[?:~]|[-+*/%&|^!=<>]=?/}),Prism.languages.insertBefore("c","string",{char:{pattern:/'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n]){0,32}'/,greedy:!0}}),Prism.languages.insertBefore("c","string",{macro:{pattern:/(^[\t ]*)#\s*[a-z](?:[^\r\n\\/]|\/(?!\*)|\/\*(?:[^*]|\*(?!\/))*\*\/|\\(?:\r\n|[\s\S]))*/im,lookbehind:!0,greedy:!0,alias:"property",inside:{string:[{pattern:/^(#\s*include\s*)<[^>]+>/,lookbehind:!0},Prism.languages.c.string],char:Prism.languages.c.char,comment:Prism.languages.c.comment,"macro-name":[{pattern:/(^#\s*define\s+)\w+\b(?!\()/i,lookbehind:!0},{pattern:/(^#\s*define\s+)\w+\b(?=\()/i,lookbehind:!0,alias:"function"}],directive:{pattern:/^(#\s*)[a-z]+/,lookbehind:!0,alias:"keyword"},"directive-hash":/^#/,punctuation:/##|\\(?=[\r\n])/,expression:{pattern:/\S[\s\S]*/,inside:Prism.languages.c}}}}),Prism.languages.insertBefore("c","function",{constant:/\b(?:EOF|NULL|SEEK_CUR|SEEK_END|SEEK_SET|__DATE__|__FILE__|__LINE__|__TIMESTAMP__|__TIME__|__func__|stderr|stdin|stdout)\b/}),delete Prism.languages.c.boolean;(function(e){var n=/[*&][^\s[\]{},]+/,t=/!(?:<[\w\-%#;/?:@&=+$,.!~*'()[\]]+>|(?:[a-zA-Z\d-]*!)?[\w\-%#;/?:@&=+$.~*'()]+)?/,a="(?:"+t.source+"(?:[ 	]+"+n.source+")?|"+n.source+"(?:[ 	]+"+t.source+")?)",d="(?:[^\\s\\x00-\\x08\\x0e-\\x1f!\"#%&'*,\\-:>?@[\\]`{|}\\x7f-\\x84\\x86-\\x9f\\ud800-\\udfff\\ufffe\\uffff]|[?:-]<PLAIN>)(?:[ 	]*(?:(?![#:])<PLAIN>|:<PLAIN>))*".replace(/<PLAIN>/g,function(){return"[^\\s\\x00-\\x08\\x0e-\\x1f,[\\]{}\\x7f-\\x84\\x86-\\x9f\\ud800-\\udfff\\ufffe\\uffff]"}),s=`"(?:[^"\\\\\r
]|\\\\.)*"|'(?:[^'\\\\\r
]|\\\\.)*'`;function r(i,c){c=(c||"").replace(/m/g,"")+"m";var o=`([:\\-,[{]\\s*(?:\\s<<prop>>[ 	]+)?)(?:<<value>>)(?=[ 	]*(?:$|,|\\]|\\}|(?:[\r
]\\s*)?#))`.replace(/<<prop>>/g,function(){return a}).replace(/<<value>>/g,function(){return i});return RegExp(o,c)}e.languages.yaml={scalar:{pattern:RegExp(`([\\-:]\\s*(?:\\s<<prop>>[ 	]+)?[|>])[ 	]*(?:((?:\r?
|\r)[ 	]+)\\S[^\r
]*(?:\\2[^\r
]+)*)`.replace(/<<prop>>/g,function(){return a})),lookbehind:!0,alias:"string"},comment:/#.*/,key:{pattern:RegExp(`((?:^|[:\\-,[{\r
?])[ 	]*(?:<<prop>>[ 	]+)?)<<key>>(?=\\s*:\\s)`.replace(/<<prop>>/g,function(){return a}).replace(/<<key>>/g,function(){return"(?:"+d+"|"+s+")"})),lookbehind:!0,greedy:!0,alias:"atrule"},directive:{pattern:/(^[ \t]*)%.+/m,lookbehind:!0,alias:"important"},datetime:{pattern:r("\\d{4}-\\d\\d?-\\d\\d?(?:[tT]|[ 	]+)\\d\\d?:\\d{2}:\\d{2}(?:\\.\\d*)?(?:[ 	]*(?:Z|[-+]\\d\\d?(?::\\d{2})?))?|\\d{4}-\\d{2}-\\d{2}|\\d\\d?:\\d{2}(?::\\d{2}(?:\\.\\d*)?)?"),lookbehind:!0,alias:"number"},boolean:{pattern:r("false|true","i"),lookbehind:!0,alias:"important"},null:{pattern:r("null|~","i"),lookbehind:!0,alias:"important"},string:{pattern:r(s),lookbehind:!0,greedy:!0},number:{pattern:r("[+-]?(?:0x[\\da-f]+|0o[0-7]+|(?:\\d+(?:\\.\\d*)?|\\.\\d+)(?:e[+-]?\\d+)?|\\.inf|\\.nan)","i"),lookbehind:!0},tag:t,important:n,punctuation:/---|[:[\]{}\-,|>?]|\.\.\./},e.languages.yml=e.languages.yaml})(Prism);(function(e){function n(s,r){e.languages[s]&&e.languages.insertBefore(s,"comment",{"doc-comment":r})}var t=e.languages.markup.tag,a={pattern:/\/\/\/.*/,greedy:!0,alias:"comment",inside:{tag:t}},d={pattern:/'''.*/,greedy:!0,alias:"comment",inside:{tag:t}};n("csharp",a),n("fsharp",a),n("vbnet",d)})(Prism);const yn=["innerHTML"],vn=H({__name:"DfMdRender",props:{content:{type:String,default:""}},setup(e){const{$md:n}=dt();return ct(()=>{Oe.highlightAll()}),pt(()=>{Oe.highlightAll()}),(t,a)=>(v(),I("div",{class:"html-md-render",innerHTML:gt(n).render(e.content)},null,8,yn))}}),xe=U(vn,[["__scopeId","data-v-6875820b"]]),wn="mdi-checkbox-marked-circle",Ve="mdi-content-copy",kn=H({name:"DfCopyClipboard",props:{text:{type:String,default:null},color:{type:String,default:null},end:{type:Boolean,default:!1},start:{type:Boolean,default:!1},size:{type:String,default:"small"}},data(){return{icon:Ve,localColor:null}},watch:{color:{immediate:!0,handler(e){this.localColor=e}}},methods:{copyText(){this.icon=wn,this.localColor="success",navigator.clipboard.writeText(this.text),setTimeout(this.reset,1e3)},reset(){this.icon=Ve,this.localColor=this.color}}});function _n(e,n,t,a,d,s){return v(),T(J,{size:e.size,onClick:e.copyText},{default:D(()=>[F(oe,{size:e.size,end:e.end,start:e.start,color:e.localColor},{default:D(()=>[le(M(e.icon),1)]),_:1},8,["size","end","start","color"])]),_:1},8,["size","onClick"])}const Sn=U(kn,[["render",_n]]),xn=H({name:"DfMarkdownCode",components:{DfMdRender:xe,DfCopyClipboard:Sn},props:{content:{type:String,required:!0}},data(){return{localContent:"",maxNumberOfContentLines:20,contentExpanded:!1}},computed:{formattedContent(){var e;return(e=this.content)==null?void 0:e.replace(/^```[^\n]*\n?|```\s*$/g,"")},showExpandContentButtons(){var e,n;return((n=(e=this.formattedContent)==null?void 0:e.split(`
`))==null?void 0:n.length)>this.maxNumberOfContentLines}},watch:{content:{immediate:!0,handler(e){this.localContent=e,this.collapseContent()}}},methods:{expandContent(){this.localContent=this.content,this.contentExpanded=!0},collapseContent(){var e,n;this.showExpandContentButtons===!0&&(this.localContent=((n=(e=this.localContent)==null?void 0:e.split(`
`))==null?void 0:n.slice(0,this.maxNumberOfContentLines).join(`
`))+"\n...\n```",this.contentExpanded=!1)}}}),$n={style:{position:"relative"}},An={style:{position:"absolute",top:"0",right:"0"},class:"d-flex align-center mt-3 mr-3"},En={key:0,class:"mr-2"};function Fn(e,n,t,a,d,s){const r=N("DfMdRender"),i=N("DfCopyClipboard");return v(),I("div",$n,[F(r,{format:"MARKDOWN",content:e.localContent},null,8,["content"]),R("div",An,[e.showExpandContentButtons?(v(),I("div",En,[e.contentExpanded===!1?(v(),T(J,{key:0,size:"x-small",style:{"margin-top":"-2px"},onClick:e.expandContent},{default:D(()=>[F(oe,{size:"x-small",color:"black"},{default:D(()=>[le(" mdi-arrow-expand ")]),_:1})]),_:1},8,["onClick"])):(v(),T(J,{key:1,size:"x-small",style:{"margin-top":"-2px"},onClick:e.collapseContent},{default:D(()=>[F(oe,{size:"x-small",color:"black"},{default:D(()=>[le(" mdi-arrow-collapse ")]),_:1})]),_:1},8,["onClick"]))])):z("",!0),F(i,{text:e.formattedContent,size:"x-small"},null,8,["text"])])])}const Qe=U(xn,[["render",Fn]]),Tn=H({name:"DfMarkdownTabs",components:{DfMdRender:xe,DfMarkdownCode:Qe},props:{tabsData:{type:Object,default:null},tabsDataIndex:{type:Number,default:0},format:{type:String,default:Ue.Markdown}},data(){return{tabs:[]}}}),$e=Symbol.for("vuetify:v-tabs"),In=de({fixed:Boolean,sliderColor:String,hideSlider:Boolean,direction:{type:String,default:"horizontal"},...qe(ft({selectedClass:"v-tab--selected",variant:"text"}),["active","block","flat","location","position","symbol"])},"VTab"),Je=ce()({name:"VTab",props:In(),setup(e,n){let{slots:t,attrs:a}=n;const{textColorClasses:d,textColorStyles:s}=ht(e,"sliderColor"),r=Fe(),i=Fe(),c=ue(()=>e.direction==="horizontal"),o=ue(()=>{var m,k;return((k=(m=r.value)==null?void 0:m.group)==null?void 0:k.isSelected.value)??!1});function p(m){var _,x;let{value:k}=m;if(k){const w=(x=(_=r.value)==null?void 0:_.$el.parentElement)==null?void 0:x.querySelector(".v-tab--selected .v-tab__slider"),S=i.value;if(!w||!S)return;const A=getComputedStyle(w).color,l=w.getBoundingClientRect(),u=S.getBoundingClientRect(),g=c.value?"x":"y",f=c.value?"X":"Y",h=c.value?"right":"bottom",b=c.value?"width":"height",$=l[g],y=u[g],E=$>y?l[h]-u[h]:l[g]-u[g],C=Math.sign(E)>0?c.value?"right":"bottom":Math.sign(E)<0?c.value?"left":"top":"center",ee=(Math.abs(E)+(Math.sign(E)<0?l[b]:u[b]))/Math.max(l[b],u[b])||0,te=l[b]/u[b]||0,ne=1.5;bt(S,{backgroundColor:[A,"currentcolor"],transform:[`translate${f}(${E}px) scale${f}(${te})`,`translate${f}(${E/ne}px) scale${f}(${(ee-1)/ne+1})`,"none"],transformOrigin:Array(3).fill(C)},{duration:225,easing:yt})}}return pe(()=>{const m=J.filterProps(e);return F(J,Z({symbol:$e,ref:r,class:["v-tab",e.class],style:e.style,tabindex:o.value?0:-1,role:"tab","aria-selected":String(o.value),active:!1},m,a,{block:e.fixed,maxWidth:e.fixed?300:void 0,"onGroup:selected":p}),{...t,default:()=>{var k;return F(V,null,[((k=t.default)==null?void 0:k.call(t))??e.text,!e.hideSlider&&F("div",{ref:i,class:["v-tab__slider",d.value],style:s.value},null)])}})}),mt({},r)}}),Dn=de({...qe(vt(),["continuous","nextIcon","prevIcon","showArrows","touch","mandatory"])},"VTabsWindow"),et=ce()({name:"VTabsWindow",props:Dn(),emits:{"update:modelValue":e=>!0},setup(e,n){let{slots:t}=n;const a=wt($e,null),d=He(e,"modelValue"),s=ue({get(){var r;return d.value!=null||!a?d.value:(r=a.items.value.find(i=>a.selected.value.includes(i.id)))==null?void 0:r.value},set(r){d.value=r}});return pe(()=>{const r=Te.filterProps(e);return F(Te,Z({_as:"VTabsWindow"},r,{modelValue:s.value,"onUpdate:modelValue":i=>s.value=i,class:["v-tabs-window",e.class],style:e.style,mandatory:!1,touch:!1}),t)}),{}}}),Cn=de({...kt()},"VTabsWindowItem"),tt=ce()({name:"VTabsWindowItem",props:Cn(),setup(e,n){let{slots:t}=n;return pe(()=>{const a=Ie.filterProps(e);return F(Ie,Z({_as:"VTabsWindowItem"},a,{class:["v-tabs-window-item",e.class],style:e.style}),t)}),{}}});function Pn(e){return e?e.map(n=>It(n)?n:{text:n,value:n}):[]}const Rn=de({alignTabs:{type:String,default:"start"},color:String,fixedTabs:Boolean,items:{type:Array,default:()=>[]},stacked:Boolean,bgColor:String,grow:Boolean,height:{type:[Number,String],default:void 0},hideSlider:Boolean,sliderColor:String,..._t({mandatory:"force",selectedClass:"v-tab-item--selected"}),...St(),...xt()},"VTabs"),zn=ce()({name:"VTabs",props:Rn(),emits:{"update:modelValue":e=>!0},setup(e,n){let{attrs:t,slots:a}=n;const d=He(e,"modelValue"),s=ue(()=>Pn(e.items)),{densityClasses:r}=$t(e),{backgroundColorClasses:i,backgroundColorStyles:c}=At(G(e,"bgColor")),{scopeId:o}=Et();return Ft({VTab:{color:G(e,"color"),direction:G(e,"direction"),stacked:G(e,"stacked"),fixed:G(e,"fixedTabs"),sliderColor:G(e,"sliderColor"),hideSlider:G(e,"hideSlider")}}),pe(()=>{const p=De.filterProps(e),m=!!(a.window||e.items.length>0);return F(V,null,[F(De,Z(p,{modelValue:d.value,"onUpdate:modelValue":k=>d.value=k,class:["v-tabs",`v-tabs--${e.direction}`,`v-tabs--align-tabs-${e.alignTabs}`,{"v-tabs--fixed-tabs":e.fixedTabs,"v-tabs--grow":e.grow,"v-tabs--stacked":e.stacked},r.value,i.value,e.class],style:[{"--v-tabs-height":Tt(e.height)},c.value,e.style],role:"tablist",symbol:$e},o,t),{default:()=>{var k;return[((k=a.default)==null?void 0:k.call(a))??s.value.map(_=>{var x;return((x=a.tab)==null?void 0:x.call(a,{item:_}))??F(Je,Z(_,{key:_.text,value:_.value}),{default:a[`tab.${_.value}`]?()=>{var w;return(w=a[`tab.${_.value}`])==null?void 0:w.call(a,{item:_})}:void 0})})]}}),m&&F(et,Z({modelValue:d.value,"onUpdate:modelValue":k=>d.value=k,key:"tabs-window"},o),{default:()=>{var k;return[s.value.map(_=>{var x;return((x=a.item)==null?void 0:x.call(a,{item:_}))??F(tt,{value:_.value},{default:()=>{var w;return(w=a[`item.${_.value}`])==null?void 0:w.call(a,{item:_})}})}),(k=a.window)==null?void 0:k.call(a)]}})])}),{}}}),Nn={class:"d-flex align-center"},On={class:"mb-0"};function Vn(e,n,t,a,d,s){const r=N("DfMdRender"),i=N("DfMarkdownCode");return e.tabsData!==null?(v(),T(Se,{key:0,variant:"outlined",flat:"",class:"markdown-tabs"},{default:D(()=>[F(zn,{modelValue:e.tabs[e.tabsDataIndex],"onUpdate:modelValue":n[0]||(n[0]=c=>e.tabs[e.tabsDataIndex]=c)},{default:D(()=>[(v(!0),I(V,null,X(e.tabsData.tabsHeaders,c=>(v(),T(Je,{key:c,style:{"text-transform":"none"},value:e.tabs[e.tabsDataIndex]},{default:D(()=>[R("div",Nn,[F(Me,{eager:"",src:`/technologies/blackandwhite/${c.split(" ").join("_").toLowerCase()}.png`,width:"15",class:"mr-2"},null,8,["src"]),R("p",On,M(c),1)])]),_:2},1032,["value"]))),128))]),_:1},8,["modelValue"]),F(et,{modelValue:e.tabs[e.tabsDataIndex],"onUpdate:modelValue":n[1]||(n[1]=c=>e.tabs[e.tabsDataIndex]=c)},{default:D(()=>[(v(!0),I(V,null,X(e.tabsData.tabsContent,c=>(v(),T(tt,{key:`tabItem-${c.length}`,transition:!1,"reverse-transition":!1,value:e.tabs[e.tabsDataIndex]},{default:D(()=>[(v(!0),I(V,null,X(c,o=>(v(),I("div",{key:`tabContent-${o.type}`},[o.type==="text"?(v(),T(r,{key:0,format:e.format,content:o.content},null,8,["format","content"])):o.type==="code"?(v(),T(i,{key:1,content:o.content},null,8,["content"])):z("",!0)]))),128))]),_:2},1032,["value"]))),128))]),_:1},8,["modelValue"])]),_:1})):z("",!0)}const Ln=U(Tn,[["render",Vn]]);var Q=(e=>(e.StartTabs="===",e.StartCode="```",e))(Q||{});const Le={"===":/^(?!\t)(?!===)\S+(?:\s+\S*)*$/,"```":/^```\s*$/},we=[/^\t/,/^```/];class W{static enhanceMarkdown(n){const t=[],a={[Q.StartTabs]:W.formatToTabs,[Q.StartCode]:W.formatToCode},d=Object.keys(Le),s=n==null?void 0:n.split(`
`);let r="text",i="",c=null;return(s||[]).forEach((o,p)=>{const m=r!=="custom"&&d.some(_=>o.startsWith(_)===!0),k=r==="custom"&&Le[c||""].test(o)===!0;if(m===!0)r==="text"&&i.trim()!==""&&(t.push({type:"text",content:i}),i=""),c=d.find(_=>o.includes(_)),r="custom",i+=o+`
`;else if(k===!0||r==="custom"&&p===s.length-1){p===s.length-1&&we.some(x=>x.test(o)===!0)===!0?i+=o+`
`:p!==s.length-1&&we.some(x=>x.test(o)===!0)===!0&&(i+=o);const _=a[c||""](i);t.push({type:_.type,..._.content}),r="text",i="",c=null,we.some(x=>x.test(o))===!1&&(i+=o+`
`)}else i+=o+`
`;p===s.length-1&&r==="text"&&i.trim()!==""&&t.push({type:"text",content:i})}),t}static formatToTabs(n){const t=n==null?void 0:n.split(`
`);let a=null;const d=[],s=[];let r="";return(t||[]).forEach(i=>{i.startsWith(Q.StartTabs)===!0?(a!=null&&r.trim()!==""&&(d.push(a),s.push(W.enhanceMarkdown(r))),a=W.formatTabId(i==null?void 0:i.split(Q.StartTabs)[1]),r=""):r+=i.replace(/^\t/,"")+`
`}),a!=null&&r.trim()!==""&&(d.push(a),s.push(W.enhanceMarkdown(r))),{type:"tabs",content:{tabsHeaders:d,tabsContent:s}}}static formatToCode(n){return{type:"code",content:{content:n}}}static formatTabId(n){var t;return(t=n==null?void 0:n.trim())==null?void 0:t.replace(/^['"](.*)['"]$/,"$1")}}const Mn=H({name:"DfEnhancedMsRender",components:{DfMarkdownTabs:Ln,DfMarkdownCode:Qe,DfMdRender:xe},props:{content:{type:String,default:""},format:{type:String,default:Ue.Markdown},isEditable:{type:Boolean,default:!1}},emits:["validateDescription","update:editDescription","update:description","resetDescriptionChanges"],data(){return{editDescription:!1,enhancedMarkdown:[],editableDescription:this.content}},watch:{content:{immediate:!0,handler(e){this.enhanceMarkdown(e),this.content==="No Description"?this.editableDescription="":this.editableDescription=this.content,this.editableDescription=this.content}},editableDescription(e){this.$emit("update:description",e)},editDescription(e){this.$emit("update:editDescription",e)}},methods:{enhanceMarkdown(e){this.enhancedMarkdown=W.enhanceMarkdown(e)},switchDescriptionToReadOnly(){this.editDescription=!1},resetDescriptionChanges(){this.$emit("resetDescriptionChanges")}}}),Bn={class:"html-md-render",style:{color:"black"}},qn={key:0},Hn={key:1,class:"hand-pointer"};function Un(e,n,t,a,d,s){const r=N("DfMdRender"),i=N("DfMarkdownTabs"),c=N("DfMarkdownCode");return v(),I("div",Bn,[e.isEditable===!1?(v(),I("div",qn,[(v(!0),I(V,null,X(e.enhancedMarkdown,(o,p)=>(v(),I("div",{key:p},[o.type==="text"?(v(),T(r,{key:0,format:e.format,content:o.content},null,8,["format","content"])):o.type==="tabs"?(v(),T(i,{key:1,"tabs-data":o,"tabs-data-index":p,class:"my-3"},null,8,["tabs-data","tabs-data-index"])):o.type==="code"?(v(),T(c,{key:2,content:o.content},null,8,["content"])):z("",!0)]))),128))])):(v(),I("div",Hn,[Ce(R("div",{onDblclick:n[0]||(n[0]=o=>e.editDescription=!0)},[(v(!0),I(V,null,X(e.enhancedMarkdown,(o,p)=>(v(),I("div",{key:p},[o.type==="text"?(v(),T(r,{key:0,format:e.format,content:o.content},null,8,["format","content"])):o.type==="tabs"?(v(),T(i,{key:1,format:e.format,"tabs-data":o,"tabs-data-index":p,class:"my-3"},null,8,["format","tabs-data","tabs-data-index"])):o.type==="code"?(v(),T(c,{key:2,content:o.content},null,8,["content"])):z("",!0)]))),128))],544),[[Pe,e.editDescription===!1]]),Ce(F(Lt,{modelValue:e.editableDescription,"onUpdate:modelValue":n[1]||(n[1]=o=>e.editableDescription=o),"auto-grow":"","full-width":"",variant:"outlined",density:"compact",rows:"15",color:"accent",name:"Description",label:"Description",counter:"",autofocus:"",onKeydown:Re(e.resetDescriptionChanges,["esc"]),onKeyup:n[2]||(n[2]=Re(Dt(o=>e.$emit("validateDescription"),["alt"]),["enter"]))},null,8,["modelValue","onKeydown"]),[[Pe,e.editDescription===!0]])]))])}const jn=U(Mn,[["render",Un]]),ke=1,Gn={info:1,important:2,secure:3,hardening:4,potentially:5,low:6,medium:7,high:8,critical:9},Wn=H({name:"VulnerabilityDetail",components:{VulnerabilityTitle:rn,DfMaterialCard:mn,DfEnhancedMdRender:jn,DfBreadcrumbs:Nt},props:{scanId:{type:[Number,String],required:!0},vulnTitle:{type:String,required:!0},scanner:{type:Object,required:!0},showBreadcrumbs:{type:Boolean,default:!0}},emits:["update:loading"],data(){return{scansService:new Mt(this.$axios),currentHeight:0,loading:!0,vulnerabilityService:new ye(this.$axios),mutex:new Jt,getVulnInfo:!0,references:{},vulnz:[],itemsPerPage:10,currentPage:1,countVulnerabilities:0,highestRiskRating:null,title:null,description:null,recommendation:null,breadcrumbs:[{text:"Scans",disabled:!1,to:"/scan/list",exact:!0},{text:"Scan",disabled:!1,exact:!0,scans:[]},{text:"Vulnerability",scanId:this.scanId,exact:!0,kbvulnerabilities:[]}]}},computed:{vulnerabilitiesCountText(){return this.countVulnerabilities===0||this.countVulnerabilities>1?`${this.countVulnerabilities} issues`:`${this.countVulnerabilities} issue`}},watch:{async vulnTitle(){this.currentPage=1,this.getVulnInfo=!0,this.vulnz=[],this.references={},await this.fetchData()},loading(e){this.$emit("update:loading",e)}},async mounted(){addEventListener("scroll",this.handleScroll),await this.fetchData()},beforeUnmount(){removeEventListener("scroll",this.handleScroll)},methods:{...Ct(Ot,["reportError"]),async fetchData(){this.vulnerabilityService=new ye(this.$axios),await this.getDataFromApi(),await Promise.allSettled([this.getDataFromApi(),this.fetchKBVulnerabilities(),this.fetchScans()])},async getDataFromApi(){var n,t,a,d,s,r,i;const e=await this.mutex.acquire();try{if(this.vulnerabilityService.hasNext===!0){if(this.loading=!0,this.getVulnInfo===!0){const c=await this.vulnerabilityService.getVulnerabilitiesWithInfo({scanner:this.scanner,scanId:parseInt(this.scanId),vulnDetailTitles:[this.vulnTitle],kbDetailTitle:this.vulnTitle,page:this.currentPage,numberElements:this.itemsPerPage}),o=((n=(c||[])[0])==null?void 0:n.vulnerabilities)||{pageInfo:{},vulnerabilities:[]};if(this.countVulnerabilities=((t=o==null?void 0:o.pageInfo)==null?void 0:t.count)||0,((o==null?void 0:o.vulnerabilities)||[]).length>0){const p=o==null?void 0:o.vulnerabilities[0];this.highestRiskRating=(a=(c||[])[0])==null?void 0:a.highestRiskRating,this.title=(d=p==null?void 0:p.detail)==null?void 0:d.title,this.description=(s=p==null?void 0:p.detail)==null?void 0:s.description,this.recommendation=(r=p==null?void 0:p.detail)==null?void 0:r.recommendation;for(const m of((i=p==null?void 0:p.detail)==null?void 0:i.references)||[])(m==null?void 0:m.title)!==void 0&&(m==null?void 0:m.title)!==null&&(this.references[m==null?void 0:m.title]=m==null?void 0:m.url)}for(const p of(o==null?void 0:o.vulnerabilities)||[])this.vulnz.push({id:p==null?void 0:p.id,content:p==null?void 0:p.technicalDetail});this.getVulnInfo=!1}else{const c=await this.vulnerabilityService.getVulnerabilitiesWithoutInfo({scanner:this.scanner,scanId:parseInt(this.scanId),detailTitles:[this.vulnTitle],page:this.currentPage,numberElements:this.itemsPerPage});for(const o of c)this.vulnz.push({id:o==null?void 0:o.id,content:o==null?void 0:o.technicalDetail})}this.currentPage=this.currentPage+1}}finally{this.loading=!1,e()}},async fetchKBVulnerabilities(){var e,n,t;try{let a=[];const d=await this.vulnerabilityService.getKBVulnerabilities(this.scanner,parseInt(this.scanId));for(const s of(d==null?void 0:d.kbVulnerabilities)||[])a.push({key:(e=s==null?void 0:s.kb)==null?void 0:e.title,goToVulnId:!1,risk:s==null?void 0:s.highestRiskRating,title:(n=s==null?void 0:s.kb)==null?void 0:n.title,description:(t=s==null?void 0:s.kb)==null?void 0:t.shortDescription,kb:s==null?void 0:s.kb});a=Wt(a).by([{desc:s=>{var r;return Gn[(r=s==null?void 0:s.risk)==null?void 0:r.toLocaleLowerCase()]}}]),this.breadcrumbs[2].kbvulnerabilities=a}catch(a){this.reportError((a==null?void 0:a.message)||"Error fetching scan.")}},async handleScroll(){document.documentElement.scrollTop+window.innerHeight>document.documentElement.offsetHeight-300&&this.currentHeight!==document.documentElement.offsetHeight&&this.vulnerabilityService.hasNext===!0&&await this.getDataFromApi()},async reloadData(){return this.vulnerabilityService=new ye(this.$axios),this.currentPage=1,this.vulnz=[],this.getVulnInfo=!0,await this.getDataFromApi(),{vulnz:this.vulnz,riskRating:this.highestRiskRating}},async fetchScans(){this.breadcrumbs[1].scans=await this.scansService.getScans(this.scanner,{page:1,numberElements:20}),await this.updateBreadcrumbs()},async updateBreadcrumbs(){var e,n;try{if((((e=this.breadcrumbs[ke])==null?void 0:e.scans)||[]).find(a=>a.id===this.scanId)===void 0){const a=await this.scansService.getScan(this.scanner,this.scanId);this.breadcrumbs[ke].scans=[...((n=this.breadcrumbs[ke])==null?void 0:n.scans)||[],a]}}catch{}}}}),Zn=["href"],Xn={class:"pa-0 ma-0 mr-1 grey--text"},Yn={class:"font-weight-bold d-flex align-center"},Kn={class:"mb-0"};function Qn(e,n,t,a,d,s){const r=N("DfBreadcrumbs"),i=N("VulnerabilityTitle"),c=N("DfEnhancedMdRender"),o=N("DfMaterialCard");return v(),I("div",null,[e.showBreadcrumbs?(v(),T(r,{key:0,"scan-id":e.scanId,"vuln-title":e.vulnTitle,breadcrumbs:e.breadcrumbs,scanner:e.scanner,class:"mb-5"},null,8,["scan-id","vuln-title","breadcrumbs","scanner"])):z("",!0),F(i,{title:e.title,"risk-rating":e.highestRiskRating,loading:e.loading},null,8,["title","risk-rating","loading"]),F(o,{color:"grey darken-1",light:"","max-width":"100%",class:"my-12",icon:"mdi-bug-outline",title:"Description"},{default:D(()=>[F(c,{content:e.description},null,8,["content"])]),_:1}),F(o,{color:"green lighten-1",light:"","max-width":"100%",class:"my-12",icon:"mdi-wrench-outline",title:"Recommendation"},{default:D(()=>[F(c,{content:e.recommendation},null,8,["content"])]),_:1}),e.references!==null?(v(),T(o,{key:1,color:"blue lighten-1",light:"","max-width":"100%",class:"my-12",icon:"mdi-link",title:"References"},{default:D(()=>[R("ul",null,[(v(!0),I(V,null,X(e.references,(p,m)=>(v(),I("li",{key:m},[R("a",{href:p,target:"_blank"},M(m),9,Zn)]))),128))])]),_:1})):z("",!0),F(o,{color:"red lighten-1",light:"","max-width":"100%",class:"my-12",icon:"mdi-code-braces",title:"Technical Details"},{"title-extra-info":D(()=>[R("p",Xn," Has "+M(e.vulnerabilitiesCountText),1)]),default:D(()=>[(v(!0),I(V,null,X(e.vulnz,(p,m)=>(v(),I(V,{key:`issue-${p.id}-${m}`},[R("div",Yn,[R("p",Kn," Issue #"+M(m+1),1)]),F(c,{content:p.content},null,8,["content"]),m!==e.vulnz.length-1?(v(),T(Be,{key:m,class:"mt-7 mb-4"})):z("",!0)],64))),128))]),_:1})])}const la=U(Wn,[["render",Qn]]);export{la as V,ye as a,Wt as s};
