.oxo-loader-bar[data-v-f6b6d8b3]{animation:gradient-f6b6d8b3 3s infinite;animation-fill-mode:forwards;animation-timing-function:linear;background:repeating-linear-gradient(90deg,#00415d 0,#14506a,#148bbf,#4078c0 75%,#0081ba);background-position:0 0;background-size:200% auto;bottom:-50px;filter:blur(100px);height:100px;left:-50px;position:fixed;right:-50px}@keyframes gradient-f6b6d8b3{0%{background-position:0 0}to{background-position:-200% 0}}
