import{$ as S,a1 as h,bZ as O,a3 as x,a4 as T,aF as p,a6 as k,j as e,r as w,a7 as A,ar as C,c0 as u,z as F,ab as d}from"./D8zG9Ut8.js";const I=S({id:String,text:String,...h(O({closeOnBack:!1,location:"end",locationStrategy:"connected",eager:!0,minWidth:0,offset:10,openOnClick:!1,openOnHover:!0,origin:"auto",scrim:!1,scrollStrategy:"reposition",transition:!1}),["absolute","persistent"])},"VTooltip"),U=x()({name:"VTooltip",props:I(),emits:{"update:modelValue":t=>!0},setup(t,v){let{slots:a}=v;const n=T(t,"modelValue"),{scopeId:g}=p(),f=k(),r=e(()=>t.id||`v-tooltip-${f}`),l=w(),m=e(()=>t.location.split(" ").length>1?t.location:t.location+" center"),V=e(()=>t.origin==="auto"||t.origin==="overlap"||t.origin.split(" ").length>1||t.location.split(" ").length>1?t.origin:t.origin+" center"),P=e(()=>t.transition?t.transition:n.value?"scale-transition":"fade-transition"),b=e(()=>d({"aria-describedby":r.value},t.activatorProps));return A(()=>{const y=u.filterProps(t);return F(u,d({ref:l,class:["v-tooltip",t.class],style:t.style,id:r.value},y,{modelValue:n.value,"onUpdate:modelValue":o=>n.value=o,transition:P.value,absolute:!0,location:m.value,origin:V.value,persistent:!0,role:"tooltip",activatorProps:b.value,_disableGlobalStack:!0},g),{activator:a.activator,default:function(){var c;for(var o=arguments.length,s=new Array(o),i=0;i<o;i++)s[i]=arguments[i];return((c=a.default)==null?void 0:c.call(a,...s))??t.text}})}),C({},l)}});export{U as V};
