import{d as o,L as s,af as n,M as a,i,N as p,t as m}from"./D8zG9Ut8.js";import{e as c,u}from"./CThvs1cs.js";import{u as l}from"./B_NqmWCc.js";import{V as d}from"./DjllWBf7.js";import{_ as f}from"./DlAUqK2U.js";import"./CcWp78UG.js";import"./QtiAwKG4.js";import"./CcxfNGfZ.js";import"./BRjp3Mpn.js";import"./DkCZQGJD.js";import"./BvH6OxSR.js";const _=o({name:"Vulnerability",components:{VulnerabilityDetail:d},data(){return{scanner:{endpoint:"",name:"",apiKey:""}}},computed:{...s(l,["scanners"])},mounted(){const e=this.scanners.find(r=>c(r.endpoint)===(this._.provides[n]||this.$route).params.scanner);e===void 0?(this.reportError("Scanner not found"),this.$router.push({name:"scan-list"})):this.scanner=e},methods:{...a(u,["reportError"])}});function h(e,r,v,$,y,S){const t=i("VulnerabilityDetail");return m(),p(t,{"scan-id":(e._.provides[n]||e.$route).params.scan,scanner:e.scanner,"vuln-title":(e._.provides[n]||e.$route).params.vuln},null,8,["scan-id","scanner","vuln-title"])}const L=f(_,[["render",h]]);export{L as default};
