.vuln-details-page .v-skeleton-loader__bone{margin:6px 4px!important}.v-card--material__avatar{margin-bottom:-32px;position:relative;top:-64px}.v-card--material__heading{position:relative;top:-40px;transition:.3s ease;z-index:1}.v-card--material.v-card--material--hover-reveal:hover .v-card--material__heading{transform:translateY(-40px)}ul{margin-left:1.2rem}code[class*=language-],pre[class*=language-]{background:none;color:#ccc;font-family:Consolas,Monaco,Andale Mono,Ubuntu Mono,monospace;text-align:left;white-space:pre;word-break:normal;word-spacing:normal;word-wrap:normal;-webkit-hyphens:none;hyphens:none;line-height:1.5;-moz-tab-size:4;-o-tab-size:4;tab-size:4}pre[class*=language-]{margin:.5em 0;overflow:auto;padding:1em}:not(pre)>code[class*=language-],pre[class*=language-]{background:#2d2d2d}:not(pre)>code[class*=language-]{border-radius:.3em;padding:.1em;white-space:normal}.token.block-comment,.token.cdata,.token.comment,.token.doctype,.token.prolog{color:#999}.token.punctuation{color:#ccc}.token.attr-name,.token.deleted,.token.namespace,.token.tag{color:#e2777a}.token.function-name{color:#6196cc}.token.boolean,.token.function,.token.number{color:#f08d49}.token.class-name,.token.constant,.token.property,.token.symbol{color:#f8c555}.token.atrule,.token.builtin,.token.important,.token.keyword,.token.selector{color:#cc99cd}.token.attr-value,.token.char,.token.regex,.token.string,.token.variable{color:#7ec699}.token.entity,.token.operator,.token.url{color:#67cdcc}.token.bold,.token.important{font-weight:700}.token.italic{font-style:italic}.token.entity{cursor:help}.token.inserted{color:green}#renderer-margin p[data-v-6875820b]{margin-bottom:0}.html-md-render[data-v-6875820b]{overflow-x:auto}.html-md-render pre[data-v-6875820b]{word-wrap:break-word;white-space:pre-wrap}.html-md-render img[data-v-6875820b]{max-width:100%}.hand-pointer[data-v-6875820b]{cursor:pointer}[data-v-6875820b] table{border-spacing:0;width:100%}[data-v-6875820b] th{padding:8px}[data-v-6875820b] td{border:1px solid #ccc;padding:8px}[data-v-6875820b] tr:nth-child(2n){background-color:#f9f9f9}.markdown-tabs pre{margin:0!important}.v-tab.v-tab.v-btn{border-radius:0;height:var(--v-tabs-height);min-width:90px}.v-slide-group--horizontal .v-tab{max-width:360px}.v-slide-group--vertical .v-tab{justify-content:start}.v-tab__slider{background:currentColor;bottom:0;height:2px;left:0;opacity:0;pointer-events:none;position:absolute;width:100%}.v-tab--selected .v-tab__slider{opacity:1}.v-slide-group--vertical .v-tab__slider{height:100%;top:0;width:2px}.v-tabs{display:flex;height:var(--v-tabs-height)}.v-tabs--density-default{--v-tabs-height:48px}.v-tabs--density-default.v-tabs--stacked{--v-tabs-height:72px}.v-tabs--density-comfortable{--v-tabs-height:44px}.v-tabs--density-comfortable.v-tabs--stacked{--v-tabs-height:68px}.v-tabs--density-compact{--v-tabs-height:36px}.v-tabs--density-compact.v-tabs--stacked{--v-tabs-height:60px}.v-tabs.v-slide-group--vertical{flex:none;height:auto;--v-tabs-height:48px}.v-tabs--align-tabs-title:not(.v-slide-group--has-affixes) .v-tab:first-child{margin-inline-start:42px}.v-tabs--align-tabs-center .v-slide-group__content>:last-child,.v-tabs--fixed-tabs .v-slide-group__content>:last-child{margin-inline-end:auto}.v-tabs--align-tabs-center .v-slide-group__content>:first-child,.v-tabs--fixed-tabs .v-slide-group__content>:first-child{margin-inline-start:auto}.v-tabs--grow{flex-grow:1}.v-tabs--grow .v-tab{flex:1 0 auto;max-width:none}.v-tabs--align-tabs-end .v-tab:first-child{margin-inline-start:auto}.v-tabs--align-tabs-end .v-tab:last-child{margin-inline-end:0}@media (max-width:1279.98px){.v-tabs.v-slide-group--is-overflowing.v-slide-group--horizontal:not(.v-slide-group--has-affixes) .v-tab:first-child{margin-inline-start:52px}.v-tabs.v-slide-group--is-overflowing.v-slide-group--horizontal:not(.v-slide-group--has-affixes) .v-tab:last-child{margin-inline-end:52px}}.html-md-render p{margin-bottom:0}.html-md-render pre{word-wrap:break-word;white-space:pre-wrap}.html-md-render img{max-width:100%}.hand-pointer{cursor:pointer}::v-deep table{border-spacing:0;width:100%}::v-deep th{padding:8px}::v-deep td{border:1px solid #ccc;padding:8px}::v-deep tr:nth-child(2n){background-color:#f9f9f9}
