import{aW as t,v as e,t as o,x as s}from"./D8zG9Ut8.js";import{_ as c}from"./DlAUqK2U.js";const n=t("/ripples_loading.svg"),r={},a={style:{position:"absolute",top:"50%",left:"50%",transform:"translate(-50%, -50%)"}},_=s("img",{class:"d-block",style:{height:"200px"},src:n},null,-1),i=s("p",{class:"text-center"}," Loading... ",-1),l=[_,i];function p(d,m){return o(),e("div",a,l)}const g=c(r,[["render",p]]);export{g as S};
